@@ -2,17 +2,11 @@
 Introduction
 ============
 
-SqlSoup provides a convenient way to access database tables without
-having to declare table or mapper classes ahead of time.
+SqlSoup provides a convenient way to access existing database tables without
+having to declare table or mapper classes ahead of time.  It is built on top of the SQLAlchemy ORM and provides a super-minimalistic interface to an existing database.   
 
 Suppose we have a database with users, books, and loans tables
-(corresponding to the PyWebOff dataset, if you're curious).  For
-testing purposes, we'll create this db as follows::
-
-    >>> from sqlalchemy import create_engine
-    >>> e = create_engine('sqlite:///:memory:')
-    >>> for sql in _testsql: e.execute(sql) #doctest: +ELLIPSIS
-    <...
+(corresponding to the PyWebOff dataset, if you're curious).  
 
 Creating a SqlSoup gateway is just like creating an SQLAlchemy
 engine::
@@ -20,15 +14,14 @@ engine::
     >>> from sqlalchemy.ext.sqlsoup import SqlSoup
     >>> db = SqlSoup('sqlite:///:memory:')
 
-or, you can re-use an existing metadata or engine::
+or, you can re-use an existing engine::
 
-    >>> db = SqlSoup(MetaData(e))
+    >>> db = SqlSoup(engine)
 
 You can optionally specify a schema within the database for your
 SqlSoup::
 
-    # >>> db.schema = myschemaname
-
+    >>> db.schema = myschemaname
 
 Loading objects
 ===============
@@ -38,12 +31,12 @@ Loading objects is as easy as this::
     >>> users = db.users.all()
     >>> users.sort()
     >>> users
-    [MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0), MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)]
+    [MappedUsers(name=u'Joe Student',email=u'student@example.edu',password=u'student',classname=None,admin=0), MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1)]
 
 Of course, letting the database do the sort is better::
 
     >>> db.users.order_by(db.users.name).all()
-    [MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1), MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0)]
+    [MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1), MappedUsers(name=u'Joe Student',email=u'student@example.edu',password=u'student',classname=None,admin=0)]
 
 Field access is intuitive::
 
@@ -57,24 +50,24 @@ it::
     >>> from sqlalchemy import or_, and_, desc
     >>> where = or_(db.users.name=='Bhargan Basepair', db.users.email=='student@example.edu')
     >>> db.users.filter(where).order_by(desc(db.users.name)).all()
-    [MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0), MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)]
+    [MappedUsers(name=u'Joe Student',email=u'student@example.edu',password=u'student',classname=None,admin=0), MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1)]
 
 You can also use .first() (to retrieve only the first object from a query) or
 .one() (like .first when you expect exactly one user -- it will raise an
 exception if more were returned)::
 
     >>> db.users.filter(db.users.name=='Bhargan Basepair').one()
-    MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)
+    MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1)
 
 Since name is the primary key, this is equivalent to
 
     >>> db.users.get('Bhargan Basepair')
-    MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)
+    MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1)
 
 This is also equivalent to
 
     >>> db.users.filter_by(name='Bhargan Basepair').one()
-    MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)
+    MappedUsers(name=u'Bhargan Basepair',email=u'basepair@example.edu',password=u'basepair',classname=None,admin=1)
 
 filter_by is like filter, but takes kwargs instead of full clause expressions.
 This makes it more concise for simple queries like this, but you can't do
@@ -84,10 +77,7 @@ Full query documentation
 ------------------------
 
 Get, filter, filter_by, order_by, limit, and the rest of the
-query methods are explained in detail in the `SQLAlchemy documentation`__.
-
-__ http://www.sqlalchemy.org/docs/04/ormtutorial.html#datamapping_querying
-
+query methods are explained in detail in :ref:`ormtutorial_querying`.
 
 Modifying objects
 =================
@@ -96,23 +86,22 @@ Modifying objects is intuitive::
 
     >>> user = _
     >>> user.email = 'basepair+nospam@example.edu'
-    >>> db.flush()
+    >>> db.commit()
 
 (SqlSoup leverages the sophisticated SQLAlchemy unit-of-work code, so
 multiple updates to a single object will be turned into a single
-``UPDATE`` statement when you flush.)
+``UPDATE`` statement when you commit.)
 
 To finish covering the basics, let's insert a new loan, then delete
 it::
 
     >>> book_id = db.books.filter_by(title='Regional Variation in Moss').first().id
     >>> db.loans.insert(book_id=book_id, user_name=user.name)
-    MappedLoans(book_id=2,user_name='Bhargan Basepair',loan_date=None)
-    >>> db.flush()
+    MappedLoans(book_id=2,user_name=u'Bhargan Basepair',loan_date=None)
 
     >>> loan = db.loans.filter_by(book_id=2, user_name='Bhargan Basepair').one()
     >>> db.delete(loan)
-    >>> db.flush()
+    >>> db.commit()
 
 You can also delete rows that have not been loaded as objects. Let's
 do our insert/delete cycle once more, this time using the loans
@@ -124,8 +113,7 @@ to the select methods.
 ::
 
     >>> db.loans.insert(book_id=book_id, user_name=user.name)
-    MappedLoans(book_id=2,user_name='Bhargan Basepair',loan_date=None)
-    >>> db.flush()
+    MappedLoans(book_id=2,user_name=u'Bhargan Basepair',loan_date=None)
     >>> db.loans.delete(db.loans.book_id==2)
 
 You can similarly update multiple rows at once. This will change the
@@ -133,7 +121,7 @@ book_id to 1 in all loans whose book_id is 2::
 
     >>> db.loans.update(db.loans.book_id==2, book_id=1)
     >>> db.loans.filter_by(book_id=1).all()
-    [MappedLoans(book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
+    [MappedLoans(book_id=1,user_name=u'Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
 
 
 Joins
@@ -150,7 +138,7 @@ uses that as the join condition automatically.
 
     >>> join1 = db.join(db.users, db.loans, isouter=True)
     >>> join1.filter_by(name='Joe Student').all()
-    [MappedJoin(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0,book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
+    [MappedJoin(name=u'Joe Student',email=u'student@example.edu',password=u'student',classname=None,admin=0,book_id=1,user_name=u'Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
 
 If you're unfortunate enough to be using MySQL with the default MyISAM
 storage engine, you'll have to specify the join condition manually,
@@ -166,7 +154,7 @@ books table::
 
     >>> join2 = db.join(join1, db.books)
     >>> join2.all()
-    [MappedJoin(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0,book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0),id=1,title='Mustards I Have Known',published_year='1989',authors='Jones')]
+    [MappedJoin(name=u'Joe Student',email=u'student@example.edu',password=u'student',classname=None,admin=0,book_id=1,user_name=u'Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0),id=1,title=u'Mustards I Have Known',published_year=u'1989',authors=u'Jones')]
 
 If you join tables that have an identical column name, wrap your join
 with `with_labels`, to disambiguate columns with their table name
@@ -182,44 +170,68 @@ You can also join directly to a labeled object::
     [u'name', u'email', u'password', u'classname', u'admin', u'loans_book_id', u'loans_user_name', u'loans_loan_date']
 
 
-Relations
-=========
+Relationships
+=============
 
-You can define relations on SqlSoup classes:
+You can define relationships on SqlSoup classes:
 
     >>> db.users.relate('loans', db.loans)
 
 These can then be used like a normal SA property:
 
     >>> db.users.get('Joe Student').loans
-    [MappedLoans(book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
+    [MappedLoans(book_id=1,user_name=u'Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
 
     >>> db.users.filter(~db.users.loans.any()).all()
-    [MappedUsers(name='Bhargan Basepair',email='basepair+nospam@example.edu',password='basepair',classname=None,admin=1)]
+    [MappedUsers(name=u'Bhargan Basepair',email='basepair+nospam@example.edu',password=u'basepair',classname=None,admin=1)]
 
 
-relate can take any options that the relation function accepts in normal mapper definition:
+relate can take any options that the relationship function accepts in normal mapper definition:
 
     >>> del db._cache['users']
     >>> db.users.relate('loans', db.loans, order_by=db.loans.loan_date, cascade='all, delete-orphan')
 
-
 Advanced Use
 ============
 
-Accessing the Session
----------------------
+Sessions, Transations and Application Integration
+-------------------------------------------------
 
-SqlSoup uses a SessionContext to provide thread-local sessions.  You
+**Note:** please read and understand this section thoroughly before using SqlSoup in any web application.
+
+SqlSoup uses a ScopedSession to provide thread-local sessions.  You
 can get a reference to the current one like this::
 
-    >>> from sqlalchemy.ext.sqlsoup import objectstore
-    >>> session = objectstore.current
+    >>> session = db.session
 
-Now you have access to all the standard session-based SA features,
-such as transactions.  (SqlSoup's ``flush()`` is normally
-transactionalized, but you can perform manual transaction management
-if you need a transaction to span multiple flushes.)
+The default session is available at the module level in SQLSoup, via::
+
+    >>> from sqlalchemy.ext.sqlsoup import Session
+    
+The configuration of this session is ``autoflush=True``, ``autocommit=False``.
+This means when you work with the SqlSoup object, you need to call ``db.commit()``
+in order to have changes persisted.   You may also call ``db.rollback()`` to
+roll things back.
+
+Since the SqlSoup object's Session automatically enters into a transaction as soon 
+as it's used, it is *essential* that you call ``commit()`` or ``rollback()``
+on it when the work within a thread completes.  This means all the guidelines
+for web application integration at :ref:`session_lifespan` must be followed.
+
+The SqlSoup object can have any session or scoped session configured onto it.
+This is of key importance when integrating with existing code or frameworks
+such as Pylons.   If your application already has a ``Session`` configured,
+pass it to your SqlSoup object::
+
+    >>> from myapplication import Session
+    >>> db = SqlSoup(session=Session)
+
+If the ``Session`` is configured with ``autocommit=True``, use ``flush()`` 
+instead of ``commit()`` to persist changes - in this case, the ``Session``
+closes out its transaction immediately and no external management is needed.  ``rollback()`` is also not available.  Configuring a new SQLSoup object in "autocommit" mode looks like::
+
+    >>> from sqlalchemy.orm import scoped_session, sessionmaker
+    >>> db = SqlSoup('sqlite://', session=scoped_session(sessionmaker(autoflush=False, expire_on_commit=False, autocommit=True)))
 
 
 Mapping arbitrary Selectables
@@ -241,7 +253,7 @@ PK in the database.)
     >>> s = s.alias('years_with_count')
     >>> years_with_count = db.map(s, primary_key=[s.c.published_year])
     >>> years_with_count.filter_by(published_year='1989').all()
-    [MappedBooks(published_year='1989',n=1)]
+    [MappedBooks(published_year=u'1989',n=1)]
 
 Obviously if we just wanted to get a list of counts associated with
 book years once, raw SQL is going to be less work. The advantage of
@@ -260,21 +272,21 @@ Python is flexible like that!
 Raw SQL
 -------
 
-SqlSoup works fine with SQLAlchemy's `text block support`__.
+SqlSoup works fine with SQLAlchemy's text construct, described in :ref:`sqlexpression_text`. 
+You can also execute textual SQL directly using the `execute()` method,
+which corresponds to the `execute()` method on the underlying `Session`.
+Expressions here are expressed like ``text()`` constructs, using named parameters
+with colons::
 
-__ http://www.sqlalchemy.org/docs/04/sqlexpression.html#sql_text
-
-You can also access the SqlSoup's `engine` attribute to compose SQL
-directly.  The engine's ``execute`` method corresponds to the one of a
-DBAPI cursor, and returns a ``ResultProxy`` that has ``fetch`` methods
-you would also see on a cursor::
-
-    >>> rp = db.bind.execute('select name, email from users order by name')
+    >>> rp = db.execute('select name, email from users where name like :name order by name', name='%Bhargan%')
     >>> for name, email in rp.fetchall(): print name, email
     Bhargan Basepair basepair+nospam@example.edu
-    Joe Student student@example.edu
 
-You can also pass this engine object to other SQLAlchemy constructs.
+Or you can get at the current transaction's connection using `connection()`.  This is the 
+raw connection object which can accept any sort of SQL expression or raw SQL string passed to the database::
+
+    >>> conn = db.connection()
+    >>> conn.execute("'select name, email from users where name like ? order by name'", '%Bhargan%')
 
 
 Dynamic table names
@@ -286,108 +298,53 @@ You can load a table whose name is specified at runtime with the entity() method
     >>> db.entity(tablename) == db.loans
     True
 
+entity() also takes an optional schema argument.  If none is specified, the
+default schema is used.
 
-Extra tests
-===========
 
-Boring tests here.  Nothing of real expository value.
-
-::
-
-    >>> db.users.filter_by(classname=None).order_by(db.users.name).all()
-    [MappedUsers(name='Bhargan Basepair',email='basepair+nospam@example.edu',password='basepair',classname=None,admin=1), MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0)]
-
-    >>> db.nopk
-    Traceback (most recent call last):
-    ...
-    PKNotFoundError: table 'nopk' does not have a primary key defined [columns: i]
-
-    >>> db.nosuchtable
-    Traceback (most recent call last):
-    ...
-    NoSuchTableError: nosuchtable
-
-    >>> years_with_count.insert(published_year='2007', n=1)
-    Traceback (most recent call last):
-    ...
-    InvalidRequestError: SQLSoup can only modify mapped Tables (found: Alias)
-
-    [tests clear()]
-    >>> db.loans.count()
-    1
-    >>> _ = db.loans.insert(book_id=1, user_name='Bhargan Basepair')
-    >>> db.clear()
-    >>> db.flush()
-    >>> db.loans.count()
-    1
 """
 
-from sqlalchemy import *
+from sqlalchemy import Table, MetaData, join
 from sqlalchemy import schema, sql
-from sqlalchemy.orm import *
-from sqlalchemy.ext.sessioncontext import SessionContext
-from sqlalchemy.exceptions import *
+from sqlalchemy.engine.base import Engine
+from sqlalchemy.orm import scoped_session, sessionmaker, mapper, \
+                            class_mapper, relationship, session,\
+                            object_session
+from sqlalchemy.orm.interfaces import MapperExtension, EXT_CONTINUE
+from sqlalchemy.exceptions import SQLAlchemyError, InvalidRequestError, ArgumentError
 from sqlalchemy.sql import expression
 
-_testsql = """
-CREATE TABLE books (
-    id                   integer PRIMARY KEY, -- auto-increments in sqlite
-    title                text NOT NULL,
-    published_year       char(4) NOT NULL,
-    authors              text NOT NULL
-);
-
-CREATE TABLE users (
-    name                 varchar(32) PRIMARY KEY,
-    email                varchar(128) NOT NULL,
-    password             varchar(128) NOT NULL,
-    classname            text,
-    admin                int NOT NULL -- 0 = false
-);
-
-CREATE TABLE loans (
-    book_id              int PRIMARY KEY REFERENCES books(id),
-    user_name            varchar(32) references users(name)
-        ON DELETE SET NULL ON UPDATE CASCADE,
-    loan_date            datetime DEFAULT current_timestamp
-);
-
-insert into users(name, email, password, admin)
-values('Bhargan Basepair', 'basepair@example.edu', 'basepair', 1);
-insert into users(name, email, password, admin)
-values('Joe Student', 'student@example.edu', 'student', 0);
-
-insert into books(title, published_year, authors)
-values('Mustards I Have Known', '1989', 'Jones');
-insert into books(title, published_year, authors)
-values('Regional Variation in Moss', '1971', 'Flim and Flam');
-
-insert into loans(book_id, user_name, loan_date)
-values (
-    (select min(id) from books),
-    (select name from users where name like 'Joe%'),
-    '2006-07-12 0:0:0')
-;
-
-CREATE TABLE nopk (
-    i                    int
-);
-""".split(';')
 
 __all__ = ['PKNotFoundError', 'SqlSoup']
 
-#
-# thread local SessionContext
-#
-class Objectstore(SessionContext):
-    def __getattr__(self, key):
-        return getattr(self.current, key)
-    def get_session(self):
-        return self.current
+Session = scoped_session(sessionmaker(autoflush=True, autocommit=False))
 
-objectstore = Objectstore(create_session)
+class AutoAdd(MapperExtension):
+    def __init__(self, scoped_session):
+        self.scoped_session = scoped_session
 
-class PKNotFoundError(SQLAlchemyError): pass
+    def instrument_class(self, mapper, class_):
+        class_.__init__ = self._default__init__(mapper)
+
+    def _default__init__(ext, mapper):
+        def __init__(self, **kwargs):
+            for key, value in kwargs.iteritems():
+                setattr(self, key, value)
+        return __init__
+
+    def init_instance(self, mapper, class_, oldinit, instance, args, kwargs):
+        session = self.scoped_session()
+        session._save_without_cascade(instance)
+        return EXT_CONTINUE
+
+    def init_failed(self, mapper, class_, oldinit, instance, args, kwargs):
+        sess = object_session(instance)
+        if sess:
+            sess.expunge(instance)
+        return EXT_CONTINUE
+
+class PKNotFoundError(SQLAlchemyError):
+    pass
 
 def _ddl_error(cls):
     msg = 'SQLSoup can only modify mapped Tables (found: %s)' \
@@ -400,13 +357,7 @@ class SelectableClassType(type):
     def insert(cls, **kwargs):
         _ddl_error(cls)
 
-    def delete(cls, *args, **kwargs):
-        _ddl_error(cls)
-
-    def update(cls, whereclause=None, values=None, **kwargs):
-        _ddl_error(cls)
-
-    def __selectable__(cls):
+    def __clause_element__(cls):
         return cls._table
 
     def __getattr__(cls, attr):
@@ -421,14 +372,8 @@ class TableClassType(SelectableClassType):
         o.__dict__.update(kwargs)
         return o
 
-    def delete(cls, *args, **kwargs):
-        cls._table.delete(*args, **kwargs).execute()
-
-    def update(cls, whereclause=None, values=None, **kwargs):
-        cls._table.update(whereclause, values).execute(**kwargs)
-
     def relate(cls, propname, *args, **kwargs):
-        class_mapper(cls)._compile_property(propname, relation(*args, **kwargs))
+        class_mapper(cls)._configure_property(propname, relationship(*args, **kwargs))
 
 def _is_outer_join(selectable):
     if not isinstance(selectable, sql.Join):
@@ -439,9 +384,9 @@ def _is_outer_join(selectable):
 
 def _selectable_name(selectable):
     if isinstance(selectable, sql.Alias):
-        return _selectable_name(selectable.selectable)
+        return _selectable_name(selectable.element)
     elif isinstance(selectable, sql.Select):
-        return ''.join([_selectable_name(s) for s in selectable.froms])
+        return ''.join(_selectable_name(s) for s in selectable.froms)
     elif isinstance(selectable, schema.Table):
         return selectable.name.capitalize()
     else:
@@ -450,106 +395,152 @@ def _selectable_name(selectable):
             x = x[1:]
         return x
 
-def class_for_table(selectable, **mapper_kwargs):
-    selectable = expression._selectable(selectable)
+def _class_for_table(session, engine, selectable, base_cls=object, **mapper_kwargs):
+    selectable = expression._clause_element_as_expr(selectable)
     mapname = 'Mapped' + _selectable_name(selectable)
+    # Py2K
+    if isinstance(mapname, unicode): 
+        engine_encoding = engine.dialect.encoding 
+        mapname = mapname.encode(engine_encoding)
+    # end Py2K
+    
     if isinstance(selectable, Table):
-        klass = TableClassType(mapname, (object,), {})
+        klass = TableClassType(mapname, (base_cls,), {})
     else:
-        klass = SelectableClassType(mapname, (object,), {})
+        klass = SelectableClassType(mapname, (base_cls,), {})
 
-    def __cmp__(self, o):
-        L = self.__class__.c.keys()
+    def _compare(self, o):
+        L = list(self.__class__.c.keys())
         L.sort()
         t1 = [getattr(self, k) for k in L]
         try:
             t2 = [getattr(o, k) for k in L]
         except AttributeError:
             raise TypeError('unable to compare with %s' % o.__class__)
-        return cmp(t1, t2)
+        return t1, t2
+    
+    # python2/python3 compatible system of 
+    # __cmp__ - __lt__ + __eq__
+    
+    def __lt__(self, o):
+        t1, t2 = _compare(self, o)
+        return t1 < t2
 
+    def __eq__(self, o):
+        t1, t2 = _compare(self, o)
+        return t1 == t2
+    
     def __repr__(self):
-        import locale
-        encoding = locale.getdefaultlocale()[1] or 'ascii'
-        L = []
-        for k in self.__class__.c.keys():
-            value = getattr(self, k, '')
-            if isinstance(value, unicode):
-                value = value.encode(encoding)
-            L.append("%s=%r" % (k, value))
+        L = ["%s=%r" % (key, getattr(self, key, ''))
+             for key in self.__class__.c.keys()]
         return '%s(%s)' % (self.__class__.__name__, ','.join(L))
-
-    for m in ['__cmp__', '__repr__']:
+        
+    for m in ['__eq__', '__repr__', '__lt__']:
         setattr(klass, m, eval(m))
     klass._table = selectable
+    klass.c = expression.ColumnCollection()
     mappr = mapper(klass,
                    selectable,
-                   extension=objectstore.mapper_extension,
-                   allow_null_pks=_is_outer_join(selectable),
+                   extension=AutoAdd(session),
                    **mapper_kwargs)
-    klass._query = Query(mappr)
+                   
+    for k in mappr.iterate_properties:
+        klass.c[k.key] = k.columns[0]
+    
+    klass._query = session.query_property()
     return klass
 
-class SqlSoup:
-    def __init__(self, *args, **kwargs):
+class SqlSoup(object):
+    def __init__(self, engine_or_metadata, base=object, **kw):
         """Initialize a new ``SqlSoup``.
 
+        `base` is the class that all created entity classes should subclass.
+
         `args` may either be an ``SQLEngine`` or a set of arguments
         suitable for passing to ``create_engine``.
         """
-
-        # meh, sometimes having method overloading instead of kwargs would be easier
-        if isinstance(args[0], MetaData):
-            args = list(args)
-            metadata = args.pop(0)
-            if args or kwargs:
-                raise ArgumentError('Extra arguments not allowed when metadata is given')
+        
+        self.session = kw.pop('session', Session)
+        self.base=base
+        
+        if isinstance(engine_or_metadata, MetaData):
+            self._metadata = engine_or_metadata
+        elif isinstance(engine_or_metadata, (basestring, Engine)):
+            self._metadata = MetaData(engine_or_metadata)
         else:
-            metadata = MetaData(*args, **kwargs)
-        self._metadata = metadata
+            raise ArgumentError("invalid engine or metadata argument %r" % engine_or_metadata)
+            
         self._cache = {}
         self.schema = None
-
+    
+    @property
     def engine(self):
         return self._metadata.bind
 
-    engine = property(engine)
     bind = engine
 
     def delete(self, *args, **kwargs):
-        objectstore.delete(*args, **kwargs)
-
+        self.session.delete(*args, **kwargs)
+    
+    def execute(self, stmt, **params):
+        return self.session.execute(sql.text(stmt, bind=self.bind), **params)
+    
+    @property
+    def _underlying_session(self):
+        if isinstance(self.session, session.Session):
+            return self.session
+        else:
+            return self.session()
+            
+    def connection(self):
+        return self._underlying_session._connection_for_bind(self.bind)
+        
     def flush(self):
-        objectstore.get_session().flush()
-
+        self.session.flush()
+    
+    def rollback(self):
+        self.session.rollback()
+        
+    def commit(self):
+        self.session.commit()
+        
     def clear(self):
-        objectstore.clear()
+        self.session.expunge_all()
+    
+    def expunge(self, *args, **kw):
+        self.session.expunge(*args, **kw)
+        
+    def expunge_all(self):
+        self.session.expunge_all()
 
     def map(self, selectable, **kwargs):
         try:
             t = self._cache[selectable]
         except KeyError:
-            t = class_for_table(selectable, **kwargs)
+            t = _class_for_table(self.session, self.engine, selectable, **kwargs)
             self._cache[selectable] = t
         return t
 
     def with_labels(self, item):
         # TODO give meaningful aliases
-        return self.map(expression._selectable(item).select(use_labels=True).alias('foo'))
+        return self.map(
+                    expression._clause_element_as_expr(item).
+                            select(use_labels=True).
+                            alias('foo'))
 
     def join(self, *args, **kwargs):
         j = join(*args, **kwargs)
         return self.map(j)
 
-    def entity(self, attr):
+    def entity(self, attr, schema=None):
         try:
             t = self._cache[attr]
-        except KeyError:
-            table = Table(attr, self._metadata, autoload=True, schema=self.schema)
+        except KeyError, ke:
+            table = Table(attr, self._metadata, autoload=True, autoload_with=self.bind, schema=schema or self.schema)
             if not table.primary_key.columns:
                 raise PKNotFoundError('table %r does not have a primary key defined [columns: %s]' % (attr, ','.join(table.c.keys())))
             if table.columns:
-                t = class_for_table(table)
+                t = _class_for_table(self.session, self.engine, table, self.base)
             else:
                 t = None
             self._cache[attr] = t
@@ -561,8 +552,3 @@ class SqlSoup:
     def __repr__(self):
         return 'SqlSoup(%r)' % self._metadata
 
-if __name__ == '__main__':
-    import logging
-    logging.basicConfig()
-    import doctest
-    doctest.testmod()
