@@ -1,5 +1,6 @@
 # interfaces.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer
+# mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -49,26 +50,67 @@ MANYTOONE = util.symbol('MANYTOONE')
 MANYTOMANY = util.symbol('MANYTOMANY')
 
 class MapperExtension(object):
-    """Base implementation for customizing Mapper behavior.
+    """Base implementation for customizing ``Mapper`` behavior.
 
-    For each method in MapperExtension, returning a result of EXT_CONTINUE
-    will allow processing to continue to the next MapperExtension in line or
-    use the default functionality if there are no other extensions.
-
-    Returning EXT_STOP will halt processing of further extensions handling
-    that method.  Some methods such as ``load`` have other return
-    requirements, see the individual documentation for details.  Other than
-    these exception cases, any return value other than EXT_CONTINUE or
-    EXT_STOP will be interpreted as equivalent to EXT_STOP.
+    New extension classes subclass ``MapperExtension`` and are specified
+    using the ``extension`` mapper() argument, which is a single
+    ``MapperExtension`` or a list of such.   A single mapper
+    can maintain a chain of ``MapperExtension`` objects.  When a
+    particular mapping event occurs, the corresponding method 
+    on each ``MapperExtension`` is invoked serially, and each method
+    has the ability to halt the chain from proceeding further.
+    
+    Each ``MapperExtension`` method returns the symbol
+    EXT_CONTINUE by default.   This symbol generally means "move
+    to the next ``MapperExtension`` for processing".  For methods
+    that return objects like translated rows or new object
+    instances, EXT_CONTINUE means the result of the method
+    should be ignored.   In some cases it's required for a 
+    default mapper activity to be performed, such as adding a 
+    new instance to a result list.
+    
+    The symbol EXT_STOP has significance within a chain
+    of ``MapperExtension`` objects that the chain will be stopped
+    when this symbol is returned.  Like EXT_CONTINUE, it also
+    has additional significance in some cases that a default
+    mapper activity will not be performed.
 
     """
     def instrument_class(self, mapper, class_):
+        """Receive a class when the mapper is first constructed, and has
+        applied instrumentation to the mapped class.
+        
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+        
+        """
         return EXT_CONTINUE
 
     def init_instance(self, mapper, class_, oldinit, instance, args, kwargs):
+        """Receive an instance when it's constructor is called.
+        
+        This method is only called during a userland construction of 
+        an object.  It is not called when an object is loaded from the
+        database.
+        
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+
+        """
         return EXT_CONTINUE
 
     def init_failed(self, mapper, class_, oldinit, instance, args, kwargs):
+        """Receive an instance when it's constructor has been called, 
+        and raised an exception.
+        
+        This method is only called during a userland construction of 
+        an object.  It is not called when an object is loaded from the
+        database.
+        
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+
+        """
         return EXT_CONTINUE
 
     def translate_row(self, mapper, context, row):
@@ -77,8 +119,15 @@ class MapperExtension(object):
 
         This is called when the mapper first receives a row, before
         the object identity or the instance itself has been derived
-        from that row.
-
+        from that row.   The given row may or may not be a 
+        ``RowProxy`` object - it will always be a dictionary-like
+        object which contains mapped columns as keys.  The 
+        returned object should also be a dictionary-like object
+        which recognizes mapped columns as keys.
+        
+        If the ultimate return value is EXT_CONTINUE, the row
+        is not translated.
+        
         """
         return EXT_CONTINUE
 
@@ -93,7 +142,7 @@ class MapperExtension(object):
           The mapper doing the operation
 
         selectcontext
-          SelectionContext corresponding to the instances() call
+          The QueryContext generated from the Query.
 
         row
           The result row from the database
@@ -107,7 +156,8 @@ class MapperExtension(object):
         """
         return EXT_CONTINUE
 
-    def append_result(self, mapper, selectcontext, row, instance, result, **flags):
+    def append_result(self, mapper, selectcontext, row, instance, 
+                        result, **flags):
         """Receive an object instance before that instance is appended
         to a result list.
 
@@ -121,7 +171,7 @@ class MapperExtension(object):
           The mapper doing the operation.
 
         selectcontext
-          SelectionContext corresponding to the instances() call.
+          The QueryContext generated from the Query.
 
         row
           The result row from the database.
@@ -134,12 +184,14 @@ class MapperExtension(object):
 
         \**flags
           extra information about the row, same as criterion in
-          ``create_row_processor()`` method of :class:`~sqlalchemy.orm.interfaces.MapperProperty`
+          ``create_row_processor()`` method of
+           :class:`~sqlalchemy.orm.interfaces.MapperProperty`
         """
 
         return EXT_CONTINUE
 
-    def populate_instance(self, mapper, selectcontext, row, instance, **flags):
+    def populate_instance(self, mapper, selectcontext, row, 
+                            instance, **flags):
         """Receive an instance before that instance has
         its attributes populated.
 
@@ -172,15 +224,18 @@ class MapperExtension(object):
         instance's lifetime.
 
         Note that during a result-row load, this method is called upon
-        the first row received for this instance. If eager loaders are
-        set to further populate collections on the instance, those
-        will *not* yet be completely loaded.
+        the first row received for this instance.  Note that some 
+        attributes and collections may or may not be loaded or even 
+        initialized, depending on what's present in the result rows.
+
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
 
         """
         return EXT_CONTINUE
 
     def before_insert(self, mapper, connection, instance):
-        """Receive an object instance before that instance is INSERTed
+        """Receive an object instance before that instance is inserted
         into its table.
 
         This is a good place to set up primary key values and such
@@ -188,141 +243,170 @@ class MapperExtension(object):
 
         Column-based attributes can be modified within this method
         which will result in the new value being inserted.  However
-        *no* changes to the overall flush plan can be made; this means
-        any collection modification or save() operations which occur
-        within this method will not take effect until the next flush
-        call.
+        *no* changes to the overall flush plan can be made, and 
+        manipulation of the ``Session`` will not have the desired effect.
+        To manipulate the ``Session`` within an extension, use 
+        ``SessionExtension``.
+
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
 
         """
 
         return EXT_CONTINUE
 
     def after_insert(self, mapper, connection, instance):
-        """Receive an object instance after that instance is INSERTed."""
+        """Receive an object instance after that instance is inserted.
+
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+        
+        """
 
         return EXT_CONTINUE
 
     def before_update(self, mapper, connection, instance):
-        """Receive an object instance before that instance is UPDATEed.
+        """Receive an object instance before that instance is updated.
 
         Note that this method is called for all instances that are marked as
         "dirty", even those which have no net changes to their column-based
-        attributes.  An object is marked as dirty when any of its column-based
+        attributes. An object is marked as dirty when any of its column-based
         attributes have a "set attribute" operation called or when any of its
-        collections are modified.  If, at update time, no column-based attributes
-        have any net changes, no UPDATE statement will be issued.  This means
-        that an instance being sent to before_update is *not* a guarantee that
-        an UPDATE statement will be issued (although you can affect the outcome
-        here).
+        collections are modified. If, at update time, no column-based
+        attributes have any net changes, no UPDATE statement will be issued.
+        This means that an instance being sent to before_update is *not* a
+        guarantee that an UPDATE statement will be issued (although you can
+        affect the outcome here).
+        
+        To detect if the column-based attributes on the object have net
+        changes, and will therefore generate an UPDATE statement, use
+        ``object_session(instance).is_modified(instance,
+        include_collections=False)``.
 
-        To detect if the column-based attributes on the object have net changes,
-        and will therefore generate an UPDATE statement, use
-        ``object_session(instance).is_modified(instance, include_collections=False)``.
+        Column-based attributes can be modified within this method
+        which will result in the new value being updated.  However
+        *no* changes to the overall flush plan can be made, and 
+        manipulation of the ``Session`` will not have the desired effect.
+        To manipulate the ``Session`` within an extension, use 
+        ``SessionExtension``.
 
-        Column-based attributes can be modified within this method which will
-        result in their being updated.  However *no* changes to the overall
-        flush plan can be made; this means any collection modification or
-        save() operations which occur within this method will not take effect
-        until the next flush call.
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
 
         """
 
         return EXT_CONTINUE
 
     def after_update(self, mapper, connection, instance):
-        """Receive an object instance after that instance is UPDATEed."""
+        """Receive an object instance after that instance is updated.
+
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+        
+        """
 
         return EXT_CONTINUE
 
     def before_delete(self, mapper, connection, instance):
-        """Receive an object instance before that instance is DELETEed.
+        """Receive an object instance before that instance is deleted.
 
-        Note that *no* changes to the overall
-        flush plan can be made here; this means any collection modification,
-        save() or delete() operations which occur within this method will
-        not take effect until the next flush call.
+        Note that *no* changes to the overall flush plan can be made
+        here; and manipulation of the ``Session`` will not have the
+        desired effect. To manipulate the ``Session`` within an
+        extension, use ``SessionExtension``.
+
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
 
         """
 
         return EXT_CONTINUE
 
     def after_delete(self, mapper, connection, instance):
-        """Receive an object instance after that instance is DELETEed."""
+        """Receive an object instance after that instance is deleted.
+        
+        The return value is only significant within the ``MapperExtension`` 
+        chain; the parent mapper's behavior isn't modified by this method.
+        
+        """
 
         return EXT_CONTINUE
 
 class SessionExtension(object):
-    """An extension hook object for Sessions.  Subclasses may be installed into a Session
-    (or sessionmaker) using the ``extension`` keyword argument.
-    """
+
+    """An extension hook object for Sessions.  Subclasses may be
+    installed into a Session (or sessionmaker) using the ``extension``
+    keyword argument. """
 
     def before_commit(self, session):
         """Execute right before commit is called.
-
-        Note that this may not be per-flush if a longer running transaction is ongoing."""
+        
+        Note that this may not be per-flush if a longer running
+        transaction is ongoing."""
 
     def after_commit(self, session):
         """Execute after a commit has occured.
-
-        Note that this may not be per-flush if a longer running transaction is ongoing."""
+        
+        Note that this may not be per-flush if a longer running
+        transaction is ongoing."""
 
     def after_rollback(self, session):
         """Execute after a rollback has occured.
+        
+        Note that this may not be per-flush if a longer running
+        transaction is ongoing."""
 
-        Note that this may not be per-flush if a longer running transaction is ongoing."""
-
-    def before_flush(self, session, flush_context, instances):
+    def before_flush( self, session, flush_context, instances):
         """Execute before flush process has started.
-
-        `instances` is an optional list of objects which were passed to the ``flush()``
-        method.
-        """
+        
+        `instances` is an optional list of objects which were passed to
+        the ``flush()`` method. """
 
     def after_flush(self, session, flush_context):
-        """Execute after flush has completed, but before commit has been called.
-
-        Note that the session's state is still in pre-flush, i.e. 'new', 'dirty',
-        and 'deleted' lists still show pre-flush state as well as the history
-        settings on instance attributes."""
+        """Execute after flush has completed, but before commit has been
+        called.
+        
+        Note that the session's state is still in pre-flush, i.e. 'new',
+        'dirty', and 'deleted' lists still show pre-flush state as well
+        as the history settings on instance attributes."""
 
     def after_flush_postexec(self, session, flush_context):
-        """Execute after flush has completed, and after the post-exec state occurs.
+        """Execute after flush has completed, and after the post-exec
+        state occurs.
+        
+        This will be when the 'new', 'dirty', and 'deleted' lists are in
+        their final state.  An actual commit() may or may not have
+        occured, depending on whether or not the flush started its own
+        transaction or participated in a larger transaction. """
 
-        This will be when the 'new', 'dirty', and 'deleted' lists are in their final
-        state.  An actual commit() may or may not have occured, depending on whether or not
-        the flush started its own transaction or participated in a larger transaction.
-        """
-
-    def after_begin(self, session, transaction, connection):
+    def after_begin( self, session, transaction, connection):
         """Execute after a transaction is begun on a connection
-
-        `transaction` is the SessionTransaction. This method is called after an
-        engine level transaction is begun on a connection.
-        """
+        
+        `transaction` is the SessionTransaction. This method is called
+        after an engine level transaction is begun on a connection. """
 
     def after_attach(self, session, instance):
         """Execute after an instance is attached to a session.
+        
+        This is called after an add, delete or merge. """
 
-        This is called after an add, delete or merge.
-        """
-
-    def after_bulk_update(self, session, query, query_context, result):
+    def after_bulk_update( self, session, query, query_context, result):
         """Execute after a bulk update operation to the session.
-
+        
         This is called after a session.query(...).update()
-
-        `query` is the query object that this update operation was called on.
-        `query_context` was the query context object.
+        
+        `query` is the query object that this update operation was
+        called on. `query_context` was the query context object.
         `result` is the result object returned from the bulk operation.
         """
 
-    def after_bulk_delete(self, session, query, query_context, result):
+    def after_bulk_delete( self, session, query, query_context, result):
         """Execute after a bulk delete operation to the session.
-
+        
         This is called after a session.query(...).delete()
-
-        `query` is the query object that this delete operation was called on.
-        `query_context` was the query context object.
+        
+        `query` is the query object that this delete operation was
+        called on. `query_context` was the query context object.
         `result` is the result object returned from the bulk operation.
         """
 
@@ -344,7 +428,8 @@ class MapperProperty(object):
         pass
 
     def create_row_processor(self, selectcontext, path, mapper, row, adapter):
-        """Return a 2-tuple consiting of two row processing functions and an instance post-processing function.
+        """Return a 2-tuple consiting of two row processing functions and 
+           an instance post-processing function.
 
         Input arguments are the query.SelectionContext and the *first*
         applicable row of a result set obtained within
@@ -359,17 +444,16 @@ class MapperProperty(object):
 
         Callables are of the following form::
 
-            def new_execute(state, dict_, row, **flags):
-                # process incoming instance state and given row.  the instance is
+            def new_execute(state, dict_, row, isnew):
+                # process incoming instance state and given row.  
+                # the instance is
                 # "new" and was just created upon receipt of this row.
-                # flags is a dictionary containing at least the following
-                # attributes:
-                #   isnew - indicates if the instance was newly created as a
-                #           result of reading this row
-                #   instancekey - identity key of the instance
+                "isnew" indicates if the instance was newly created as a
+                result of reading this row
 
-            def existing_execute(state, dict_, row, **flags):
-                # process incoming instance state and given row.  the instance is
+            def existing_execute(state, dict_, row):
+                # process incoming instance state and given row.  the 
+                # instance is
                 # "existing" and was created based on a previous row.
 
             return (new_execute, existing_execute)
@@ -380,7 +464,8 @@ class MapperProperty(object):
 
         raise NotImplementedError()
 
-    def cascade_iterator(self, type_, state, visited_instances=None, halt_on=None):
+    def cascade_iterator(self, type_, state, visited_instances=None,
+                            halt_on=None):
         """Iterate through instances related to the given instance for
         a particular 'cascade', starting with this MapperProperty.
 
@@ -408,13 +493,22 @@ class MapperProperty(object):
         self.do_init()
         self._compile_finished = True
 
+    @property
+    def class_attribute(self):
+        """Return the class-bound descriptor corresponding to this
+        MapperProperty."""
+
+        return getattr(self.parent.class_, self.key)
+
     def do_init(self):
-        """Perform subclass-specific initialization post-mapper-creation steps.
-
-        This is a *template* method called by the
-        ``MapperProperty`` object's init() method.
-
+        """Perform subclass-specific initialization post-mapper-creation
+        steps.
+        
+        This is a template method called by the ``MapperProperty``
+        object's init() method.
+        
         """
+
         pass
 
     def post_instrument_class(self, mapper):
@@ -424,26 +518,9 @@ class MapperProperty(object):
         """
         pass
 
-    def register_dependencies(self, *args, **kwargs):
-        """Called by the ``Mapper`` in response to the UnitOfWork
-        calling the ``Mapper``'s register_dependencies operation.
-        Establishes a topological dependency between two mappers
-        which will affect the order in which mappers persist data.
-        
-        """
-
+    def per_property_preprocessors(self, uow):
         pass
 
-    def register_processors(self, *args, **kwargs):
-        """Called by the ``Mapper`` in response to the UnitOfWork
-        calling the ``Mapper``'s register_processors operation.
-        Establishes a processor object between two mappers which
-        will link data and state between parent/child objects.
-        
-        """
-
-        pass
-        
     def is_primary(self):
         """Return True if this ``MapperProperty``'s mapper is the
         primary mapper for its class.
@@ -455,7 +532,7 @@ class MapperProperty(object):
 
         return not self.parent.non_primary
 
-    def merge(self, session, source, dest, dont_load, _recursive):
+    def merge(self, session, source, dest, load, _recursive):
         """Merge the attribute represented by this ``MapperProperty``
         from source to destination object"""
 
@@ -490,10 +567,11 @@ class PropComparator(expression.ColumnOperators):
         raise NotImplementedError("%r" % self)
 
     def adapted(self, adapter):
-        """Return a copy of this PropComparator which will use the given adaption function
-        on the local side of generated expressions.
-
+        """Return a copy of this PropComparator which will use the given
+        adaption function on the local side of generated expressions.
+        
         """
+
         return self.__class__(self.prop, self.mapper, adapter)
 
     @staticmethod
@@ -511,7 +589,8 @@ class PropComparator(expression.ColumnOperators):
     def of_type(self, class_):
         """Redefine this object in terms of a polymorphic subclass.
 
-        Returns a new PropComparator from which further criterion can be evaluated.
+        Returns a new PropComparator from which further criterion can be
+        evaluated.
 
         e.g.::
 
@@ -528,7 +607,8 @@ class PropComparator(expression.ColumnOperators):
         return self.operate(PropComparator.of_type_op, class_)
 
     def any(self, criterion=None, **kwargs):
-        """Return true if this collection contains any member that meets the given criterion.
+        """Return true if this collection contains any member that meets the
+        given criterion.
 
         criterion
           an optional ClauseElement formulated against the member class' table
@@ -542,7 +622,8 @@ class PropComparator(expression.ColumnOperators):
         return self.operate(PropComparator.any_op, criterion, **kwargs)
 
     def has(self, criterion=None, **kwargs):
-        """Return true if this element references a member which meets the given criterion.
+        """Return true if this element references a member which meets the
+        given criterion.
 
         criterion
           an optional ClauseElement formulated against the member class' table
@@ -560,13 +641,15 @@ class StrategizedProperty(MapperProperty):
     """A MapperProperty which uses selectable strategies to affect
     loading behavior.
 
-    There is a single default strategy selected by default.  Alternate
+    There is a single strategy selected by default.  Alternate
     strategies can be selected at Query time through the usage of
     ``StrategizedOption`` objects via the Query.options() method.
+    
     """
 
-    def __get_context_strategy(self, context, path):
-        cls = context.attributes.get(("loaderstrategy", path), None)
+    def _get_context_strategy(self, context, path):
+        cls = context.attributes.get(('loaderstrategy',
+                _reduce_path(path)), None)
         if cls:
             try:
                 return self.__all_strategies[cls]
@@ -587,10 +670,12 @@ class StrategizedProperty(MapperProperty):
         return strategy
 
     def setup(self, context, entity, path, adapter, **kwargs):
-        self.__get_context_strategy(context, path + (self.key,)).setup_query(context, entity, path, adapter, **kwargs)
+        self._get_context_strategy(context, path + (self.key,)).\
+                    setup_query(context, entity, path, adapter, **kwargs)
 
     def create_row_processor(self, context, path, mapper, row, adapter):
-        return self.__get_context_strategy(context, path + (self.key,)).create_row_processor(context, path, mapper, row, adapter)
+        return self._get_context_strategy(context, path + (self.key,)).\
+                    create_row_processor(context, path, mapper, row, adapter)
 
     def do_init(self):
         self.__all_strategies = {}
@@ -609,11 +694,11 @@ def build_path(entity, key, prev=None):
 def serialize_path(path):
     if path is None:
         return None
-
-    return [
-        (mapper.class_, key)
-        for mapper, key in [(path[i], path[i+1]) for i in range(0, len(path)-1, 2)]
-    ]
+    
+    return zip(
+        [m.class_ for m in [path[i] for i in range(0, len(path), 2)]], 
+        [path[i] for i in range(1, len(path), 2)] + [None]
+    )
 
 def deserialize_path(path):
     if path is None:
@@ -622,10 +707,11 @@ def deserialize_path(path):
     global class_mapper
     if class_mapper is None:
         from sqlalchemy.orm import class_mapper
-
-    return tuple(
-        chain(*[(class_mapper(cls), key) for cls, key in path])
-    )
+    
+    p = tuple(chain(*[(class_mapper(cls), key) for cls, key in path]))
+    if p and p[-1] is None:
+        p = p[0:-1]
+    return p
 
 class MapperOption(object):
     """Describe a modification to a Query."""
@@ -639,16 +725,19 @@ class MapperOption(object):
         pass
 
     def process_query_conditionally(self, query):
-        """same as process_query(), except that this option may not apply
-        to the given query.
-
+        """same as process_query(), except that this option may not
+        apply to the given query.
+        
         Used when secondary loaders resend existing options to a new
         Query."""
+
         self.process_query(query)
 
 class ExtensionOption(MapperOption):
-    """a MapperOption that applies a MapperExtension to a query operation."""
-    
+
+    """a MapperOption that applies a MapperExtension to a query
+    operation."""
+
     def __init__(self, ext):
         self.ext = ext
 
@@ -659,8 +748,7 @@ class ExtensionOption(MapperOption):
 
 class PropertyOption(MapperOption):
     """A MapperOption that is applied to a property off the mapper or
-    one of its child mappers, identified by a dot-separated key.
-    """
+    one of its child mappers, identified by a dot-separated key. """
 
     def __init__(self, key, mapper=None):
         self.key = key
@@ -673,31 +761,13 @@ class PropertyOption(MapperOption):
         self._process(query, False)
 
     def _process(self, query, raiseerr):
-        paths, mappers = self.__get_paths(query, raiseerr)
+        paths, mappers = self._get_paths(query, raiseerr)
         if paths:
             self.process_query_property(query, paths, mappers)
 
     def process_query_property(self, query, paths, mappers):
         pass
 
-    def __find_entity(self, query, mapper, raiseerr):
-        from sqlalchemy.orm.util import _class_to_mapper, _is_aliased_class
-
-        if _is_aliased_class(mapper):
-            searchfor = mapper
-        else:
-            searchfor = _class_to_mapper(mapper).base_mapper
-        
-        for ent in query._mapper_entities:
-            if ent.path_entity is searchfor:
-                return ent
-        else:
-            if raiseerr:
-                raise sa_exc.ArgumentError("Can't find entity %s in Query.  Current list: %r" 
-                    % (searchfor, [str(m.path_entity) for m in query._entities]))
-            else:
-                return None
-
     def __getstate__(self):
         d = self.__dict__.copy()
         d['key'] = ret = []
@@ -719,78 +789,97 @@ class PropertyOption(MapperOption):
         state['key'] = tuple(ret)
         self.__dict__ = state
 
-    def __get_paths(self, query, raiseerr):
+    def _find_entity( self, query, mapper, raiseerr):
+        from sqlalchemy.orm.util import _class_to_mapper, \
+            _is_aliased_class
+        if _is_aliased_class(mapper):
+            searchfor = mapper
+            isa = False
+        else:
+            searchfor = _class_to_mapper(mapper)
+            isa = True
+        for ent in query._mapper_entities:
+            if searchfor is ent.path_entity or isa \
+                and searchfor.common_parent(ent.path_entity):
+                return ent
+        else:
+            if raiseerr:
+                raise sa_exc.ArgumentError("Can't find entity %s in "
+                        "Query.  Current list: %r" % (searchfor,
+                        [str(m.path_entity) for m in query._entities]))
+            else:
+                return None
+
+    def _get_paths(self, query, raiseerr):
         path = None
         entity = None
         l = []
         mappers = []
-        
-        # _current_path implies we're in a secondary load
-        # with an existing path
-        current_path = list(query._current_path)
-            
-        if self.mapper:
-            entity = self.__find_entity(query, self.mapper, raiseerr)
-            mapper = entity.mapper
-            path_element = entity.path_entity
 
+        # _current_path implies we're in a secondary load with an
+        # existing path
+
+        current_path = list(query._current_path)
+        tokens = []
         for key in util.to_list(self.key):
             if isinstance(key, basestring):
-                tokens = key.split('.')
+                tokens += key.split('.')
             else:
-                tokens = [key]
-            for token in tokens:
-                if isinstance(token, basestring):
+                tokens += [key]
+        for token in tokens:
+            if isinstance(token, basestring):
+                if not entity:
+                    if current_path:
+                        if current_path[1] == token:
+                            current_path = current_path[2:]
+                            continue
+                    entity = query._entity_zero()
+                    path_element = entity.path_entity
+                    mapper = entity.mapper
+                mappers.append(mapper)
+                prop = mapper.get_property(token,
+                        resolve_synonyms=True, raiseerr=raiseerr)
+                key = token
+            elif isinstance(token, PropComparator):
+                prop = token.property
+                if not entity:
+                    if current_path:
+                        if current_path[0:2] == [token.parententity,
+                                prop.key]:
+                            current_path = current_path[2:]
+                            continue
+                    entity = self._find_entity(query,
+                            token.parententity, raiseerr)
                     if not entity:
-                        entity = query._entity_zero()
-                        path_element = entity.path_entity
-                        mapper = entity.mapper
-                    mappers.append(mapper)
-                    prop = mapper.get_property(token, resolve_synonyms=True, raiseerr=raiseerr)
-                    key = token
-                elif isinstance(token, PropComparator):
-                    prop = token.property
-                    if not entity:
-                        entity = self.__find_entity(query, token.parententity, raiseerr)
-                        if not entity:
-                            return [], []
-                        path_element = entity.path_entity
-                    mappers.append(prop.parent)
-                    key = prop.key
-                else:
-                    raise sa_exc.ArgumentError("mapper option expects string key or list of attributes")
+                        return [], []
+                    path_element = entity.path_entity
+                    mapper = entity.mapper
+                mappers.append(prop.parent)
+                key = prop.key
+            else:
+                raise sa_exc.ArgumentError('mapper option expects '
+                        'string key or list of attributes')
+            if prop is None:
+                return [], []
+            path = build_path(path_element, prop.key, path)
+            l.append(path)
+            if getattr(token, '_of_type', None):
+                path_element = mapper = token._of_type
+            else:
+                path_element = mapper = getattr(prop, 'mapper', None)
+            if path_element:
+                path_element = path_element
 
-                if current_path and key == current_path[1]:
-                    current_path = current_path[2:]
-                    continue
-                    
-                if prop is None:
-                    return [], []
-
-                path = build_path(path_element, prop.key, path)
-                l.append(path)
-                if getattr(token, '_of_type', None):
-                    path_element = mapper = token._of_type
-                else:
-                    path_element = mapper = getattr(prop, 'mapper', None)
-
-                if path_element:
-                    path_element = path_element.base_mapper
-                    
-                
-        # if current_path tokens remain, then
-        # we didn't have an exact path match.
         if current_path:
             return [], []
-            
         return l, mappers
 
 class AttributeExtension(object):
     """An event handler for individual attribute change events.
-
+    
     AttributeExtension is assembled within the descriptors associated
     with a mapped class.
-
+    
     """
 
     active_history = True
@@ -830,19 +919,41 @@ class StrategizedOption(PropertyOption):
     for an operation by a StrategizedProperty.
     """
 
-    def is_chained(self):
-        return False
+    is_chained = False
 
     def process_query_property(self, query, paths, mappers):
-        if self.is_chained():
+
+        # _get_context_strategy may receive the path in terms of a base
+        # mapper - e.g.  options(eagerload_all(Company.employees,
+        # Engineer.machines)) in the polymorphic tests leads to
+        # "(Person, 'machines')" in the path due to the mechanics of how
+        # the eager strategy builds up the path
+
+        if self.is_chained:
             for path in paths:
-                query._attributes[("loaderstrategy", path)] = self.get_strategy_class()
+                query._attributes[('loaderstrategy',
+                                  _reduce_path(path))] = \
+                    self.get_strategy_class()
         else:
-            query._attributes[("loaderstrategy", paths[-1])] = self.get_strategy_class()
+            query._attributes[('loaderstrategy',
+                              _reduce_path(paths[-1]))] = \
+                self.get_strategy_class()
 
     def get_strategy_class(self):
         raise NotImplementedError()
 
+def _reduce_path(path):
+    """Convert a (mapper, path) path to use base mappers.
+    
+    This is used to allow more open ended selection of loader strategies, i.e.
+    Mapper -> prop1 -> Subclass -> prop2, where Subclass is a sub-mapper
+    of the mapper referened by Mapper.prop1.
+    
+    """
+    return tuple([i % 2 != 0 and 
+                    path[i] or 
+                    getattr(path[i], 'base_mapper', path[i]) 
+                    for i in xrange(len(path))])
 
 class LoaderStrategy(object):
     """Describe the loading behavior of a StrategizedProperty object.
@@ -884,13 +995,13 @@ class LoaderStrategy(object):
     def setup_query(self, context, entity, path, adapter, **kwargs):
         pass
 
-    def create_row_processor(self, selectcontext, path, mapper, row, adapter):
-        """Return row processing functions which fulfill the contract specified
-        by MapperProperty.create_row_processor.
-
-        StrategizedProperty delegates its create_row_processor method directly
-        to this method.
-        """
+    def create_row_processor(self, selectcontext, path, mapper, 
+                                row, adapter):
+        """Return row processing functions which fulfill the contract
+        specified by MapperProperty.create_row_processor.
+        
+        StrategizedProperty delegates its create_row_processor method
+        directly to this method. """
 
         raise NotImplementedError()
 
