@@ -5,7 +5,8 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 import sqlalchemy.exceptions as sa_exc
-from sqlalchemy.util import ScopedRegistry, to_list, get_cls_kwargs, deprecated
+from sqlalchemy.util import ScopedRegistry, ThreadLocalRegistry, \
+                            to_list, get_cls_kwargs, deprecated
 from sqlalchemy.orm import (
     EXT_CONTINUE, MapperExtension, class_mapper, object_session
     )
@@ -29,7 +30,10 @@ class ScopedSession(object):
 
     def __init__(self, session_factory, scopefunc=None):
         self.session_factory = session_factory
-        self.registry = ScopedRegistry(session_factory, scopefunc)
+        if scopefunc:
+            self.registry = ScopedRegistry(session_factory, scopefunc)
+        else:
+            self.registry = ThreadLocalRegistry(session_factory)
         self.extension = _ScopedExt(self)
 
     def __call__(self, **kwargs):
@@ -48,6 +52,8 @@ class ScopedSession(object):
             return self.registry()
 
     def remove(self):
+        """Dispose of the current contextual session."""
+        
         if self.registry.has():
             self.registry().close()
         self.registry.clear()
@@ -57,9 +63,9 @@ class ScopedSession(object):
         "for information on how to replicate its behavior.")
     def mapper(self, *args, **kwargs):
         """return a mapper() function which associates this ScopedSession with the Mapper.
-        
+
         DEPRECATED.
-        
+
         """
 
         from sqlalchemy.orm import mapper
@@ -131,7 +137,7 @@ def makeprop(name):
     def get(self):
         return getattr(self.registry(), name)
     return property(get, set)
-for prop in ('bind', 'dirty', 'deleted', 'new', 'identity_map', 'is_active'):
+for prop in ('bind', 'dirty', 'deleted', 'new', 'identity_map', 'is_active', 'autoflush'):
     setattr(ScopedSession, prop, makeprop(prop))
 
 def clslevel(name):
@@ -171,7 +177,7 @@ class _ScopedExt(MapperExtension):
 
     def _default__init__(ext, mapper):
         def __init__(self, **kwargs):
-            for key, value in kwargs.items():
+            for key, value in kwargs.iteritems():
                 if ext.validate:
                     if not mapper.get_property(key, resolve_synonyms=False,
                                                raiseerr=False):
@@ -195,10 +201,5 @@ class _ScopedExt(MapperExtension):
         return EXT_CONTINUE
 
     def dispose_class(self, mapper, class_):
-        if hasattr(class_, '__init__') and hasattr(class_.__init__, '_oldinit'):
-            if class_.__init__._oldinit is not None:
-                class_.__init__ = class_.__init__._oldinit
-            else:
-                delattr(class_, '__init__')
         if hasattr(class_, 'query'):
             delattr(class_, 'query')
