@@ -1,3 +1,4 @@
+import warnings
 from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
 from sqlalchemy import *
 from sqlalchemy import exc as sa_exc, util
@@ -7,6 +8,7 @@ from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.test import testing, engines
 from sqlalchemy.util import function_named
 from test.orm import _base, _fixtures
+from sqlalchemy.test.schema import Table, Column
 
 class O2MTest(_base.MappedTest):
     """deals with inheritance and one-to-many relationships"""
@@ -14,8 +16,7 @@ class O2MTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global foo, bar, blub
         foo = Table('foo', metadata,
-            Column('id', Integer, Sequence('foo_seq', optional=True),
-                   primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(20)))
 
         bar = Table('bar', metadata,
@@ -27,7 +28,7 @@ class O2MTest(_base.MappedTest):
             Column('foo_id', Integer, ForeignKey('foo.id'), nullable=False),
             Column('data', String(20)))
 
-    def testbasic(self):
+    def test_basic(self):
         class Foo(object):
             def __init__(self, data=None):
                 self.data = data
@@ -46,7 +47,7 @@ class O2MTest(_base.MappedTest):
                 return "Blub id %d, data %s" % (self.id, self.data)
 
         mapper(Blub, blub, inherits=Bar, properties={
-            'parent_foo':relation(Foo)
+            'parent_foo':relationship(Foo)
         })
 
         sess = create_session()
@@ -73,9 +74,8 @@ class FalseDiscriminatorTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global t1
         t1 = Table('t1', metadata, 
-                    Column('id', Integer, primary_key=True), 
-                    Column('type', Boolean, nullable=False)
-                )
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True), 
+            Column('type', Boolean, nullable=False))
         
     def test_false_on_sub(self):
         class Foo(object):pass
@@ -108,7 +108,7 @@ class PolymorphicSynonymTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global t1, t2
         t1 = Table('t1', metadata,
-                   Column('id', Integer, primary_key=True),
+                   Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                    Column('type', String(10), nullable=False),
                    Column('info', String(255)))
         t2 = Table('t2', metadata,
@@ -141,7 +141,7 @@ class PolymorphicSynonymTest(_base.MappedTest):
         
     
 class CascadeTest(_base.MappedTest):
-    """that cascades on polymorphic relations continue
+    """that cascades on polymorphic relationships continue
     cascading along the path of the instance's mapper, not
     the base mapper."""
 
@@ -149,12 +149,12 @@ class CascadeTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global t1, t2, t3, t4
         t1= Table('t1', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(30))
             )
 
         t2 = Table('t2', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('t1id', Integer, ForeignKey('t1.id')),
             Column('type', String(30)),
             Column('data', String(30))
@@ -164,7 +164,7 @@ class CascadeTest(_base.MappedTest):
             Column('moredata', String(30)))
 
         t4 = Table('t4', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('t3id', Integer, ForeignKey('t3.id')),
             Column('data', String(30)))
 
@@ -179,11 +179,11 @@ class CascadeTest(_base.MappedTest):
             pass
 
         mapper(T1, t1, properties={
-            't2s':relation(T2, cascade="all")
+            't2s':relationship(T2, cascade="all")
         })
         mapper(T2, t2, polymorphic_on=t2.c.type, polymorphic_identity='t2')
         mapper(T3, t3, inherits=T2, polymorphic_identity='t3', properties={
-            't4s':relation(T4, cascade="all")
+            't4s':relationship(T4, cascade="all")
         })
         mapper(T4, t4)
 
@@ -209,13 +209,64 @@ class CascadeTest(_base.MappedTest):
         assert t4_1 in sess.deleted
         sess.flush()
 
+class M2OUseGetTest(_base.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('base', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('type', String(30))
+        )
+        Table('sub', metadata,
+            Column('id', Integer, ForeignKey('base.id'), primary_key=True),
+        )
+        Table('related', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('sub_id', Integer, ForeignKey('sub.id')),
+        )
+
+    @testing.resolve_artifact_names
+    def test_use_get(self):
+        # test [ticket:1186]
+        class Base(_base.BasicEntity):
+            pass
+        class Sub(Base):
+            pass
+        class Related(Base):
+            pass
+        mapper(Base, base, polymorphic_on=base.c.type, polymorphic_identity='b')
+        mapper(Sub, sub, inherits=Base, polymorphic_identity='s')
+        mapper(Related, related, properties={
+            # previously, this was needed for the comparison to occur:
+            # the 'primaryjoin' looks just like "Sub"'s "get" clause (based on the Base id),
+            # and foreign_keys since that join condition doesn't actually have any fks in it
+            #'sub':relationship(Sub, primaryjoin=base.c.id==related.c.sub_id, foreign_keys=related.c.sub_id)
+            
+            # now we can use this:
+            'sub':relationship(Sub)
+        })
+        
+        assert class_mapper(Related).get_property('sub').strategy.use_get
+        
+        sess = create_session()
+        s1 = Sub()
+        r1 = Related(sub=s1)
+        sess.add(r1)
+        sess.flush()
+        sess.expunge_all()
+
+        r1 = sess.query(Related).first()
+        s1 = sess.query(Sub).first()
+        def go():
+            assert r1.sub
+        self.assert_sql_count(testing.db, go, 0)
+        
+
 class GetTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global foo, bar, blub
         foo = Table('foo', metadata,
-            Column('id', Integer, Sequence('foo_seq', optional=True),
-                   primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('type', String(30)),
             Column('data', String(20)))
 
@@ -224,88 +275,97 @@ class GetTest(_base.MappedTest):
             Column('data', String(20)))
 
         blub = Table('blub', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('foo_id', Integer, ForeignKey('foo.id')),
             Column('bar_id', Integer, ForeignKey('bar.id')),
             Column('data', String(20)))
+    
+    @classmethod
+    def setup_classes(cls):
+        class Foo(_base.BasicEntity):
+            pass
 
-    def _create_test(polymorphic, name):
-        def test_get(self):
-            class Foo(object):
-                pass
+        class Bar(Foo):
+            pass
 
-            class Bar(Foo):
-                pass
+        class Blub(Bar):
+            pass
 
-            class Blub(Bar):
-                pass
+    def test_get_polymorphic(self):
+        self._do_get_test(True)
+    
+    def test_get_nonpolymorphic(self):
+        self._do_get_test(False)
 
-            if polymorphic:
-                mapper(Foo, foo, polymorphic_on=foo.c.type, polymorphic_identity='foo')
-                mapper(Bar, bar, inherits=Foo, polymorphic_identity='bar')
-                mapper(Blub, blub, inherits=Bar, polymorphic_identity='blub')
-            else:
-                mapper(Foo, foo)
-                mapper(Bar, bar, inherits=Foo)
-                mapper(Blub, blub, inherits=Bar)
+    @testing.resolve_artifact_names
+    def _do_get_test(self, polymorphic):
+        if polymorphic:
+            mapper(Foo, foo, polymorphic_on=foo.c.type, polymorphic_identity='foo')
+            mapper(Bar, bar, inherits=Foo, polymorphic_identity='bar')
+            mapper(Blub, blub, inherits=Bar, polymorphic_identity='blub')
+        else:
+            mapper(Foo, foo)
+            mapper(Bar, bar, inherits=Foo)
+            mapper(Blub, blub, inherits=Bar)
 
-            sess = create_session()
-            f = Foo()
-            b = Bar()
-            bl = Blub()
-            sess.add(f)
-            sess.add(b)
-            sess.add(bl)
-            sess.flush()
+        sess = create_session()
+        f = Foo()
+        b = Bar()
+        bl = Blub()
+        sess.add(f)
+        sess.add(b)
+        sess.add(bl)
+        sess.flush()
 
-            if polymorphic:
-                def go():
-                    assert sess.query(Foo).get(f.id) == f
-                    assert sess.query(Foo).get(b.id) == b
-                    assert sess.query(Foo).get(bl.id) == bl
-                    assert sess.query(Bar).get(b.id) == b
-                    assert sess.query(Bar).get(bl.id) == bl
-                    assert sess.query(Blub).get(bl.id) == bl
+        if polymorphic:
+            def go():
+                assert sess.query(Foo).get(f.id) is f
+                assert sess.query(Foo).get(b.id) is b
+                assert sess.query(Foo).get(bl.id) is bl
+                assert sess.query(Bar).get(b.id) is b
+                assert sess.query(Bar).get(bl.id) is bl
+                assert sess.query(Blub).get(bl.id) is bl
 
-                self.assert_sql_count(testing.db, go, 0)
-            else:
-                # this is testing the 'wrong' behavior of using get()
-                # polymorphically with mappers that are not configured to be
-                # polymorphic.  the important part being that get() always
-                # returns an instance of the query's type.
-                def go():
-                    assert sess.query(Foo).get(f.id) == f
+                # test class mismatches - item is present
+                # in the identity map but we requested a subclass
+                assert sess.query(Blub).get(f.id) is None
+                assert sess.query(Blub).get(b.id) is None
+                assert sess.query(Bar).get(f.id) is None
+                
+            self.assert_sql_count(testing.db, go, 0)
+        else:
+            # this is testing the 'wrong' behavior of using get()
+            # polymorphically with mappers that are not configured to be
+            # polymorphic.  the important part being that get() always
+            # returns an instance of the query's type.
+            def go():
+                assert sess.query(Foo).get(f.id) is f
 
-                    bb = sess.query(Foo).get(b.id)
-                    assert isinstance(b, Foo) and bb.id==b.id
+                bb = sess.query(Foo).get(b.id)
+                assert isinstance(b, Foo) and bb.id==b.id
 
-                    bll = sess.query(Foo).get(bl.id)
-                    assert isinstance(bll, Foo) and bll.id==bl.id
+                bll = sess.query(Foo).get(bl.id)
+                assert isinstance(bll, Foo) and bll.id==bl.id
 
-                    assert sess.query(Bar).get(b.id) == b
+                assert sess.query(Bar).get(b.id) is b
 
-                    bll = sess.query(Bar).get(bl.id)
-                    assert isinstance(bll, Bar) and bll.id == bl.id
+                bll = sess.query(Bar).get(bl.id)
+                assert isinstance(bll, Bar) and bll.id == bl.id
 
-                    assert sess.query(Blub).get(bl.id) == bl
+                assert sess.query(Blub).get(bl.id) is bl
 
-                self.assert_sql_count(testing.db, go, 3)
+            self.assert_sql_count(testing.db, go, 3)
 
-        test_get = function_named(test_get, name)
-        return test_get
-
-    test_get_polymorphic = _create_test(True, 'test_get_polymorphic')
-    test_get_nonpolymorphic = _create_test(False, 'test_get_nonpolymorphic')
 
 class EagerLazyTest(_base.MappedTest):
     """tests eager load/lazy load of child items off inheritance mappers, tests that
     LazyLoader constructs the right query condition."""
+    
     @classmethod
     def define_tables(cls, metadata):
         global foo, bar, bar_foo
         foo = Table('foo', metadata,
-                    Column('id', Integer, Sequence('foo_seq', optional=True),
-                           primary_key=True),
+                    Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                     Column('data', String(30)))
         bar = Table('bar', metadata,
                     Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
@@ -317,14 +377,14 @@ class EagerLazyTest(_base.MappedTest):
         )
 
     @testing.fails_on('maxdb', 'FIXME: unknown')
-    def testbasic(self):
+    def test_basic(self):
         class Foo(object): pass
         class Bar(Foo): pass
 
         foos = mapper(Foo, foo)
         bars = mapper(Bar, bar, inherits=foos)
-        bars.add_property('lazy', relation(foos, bar_foo, lazy=True))
-        bars.add_property('eager', relation(foos, bar_foo, lazy=False))
+        bars.add_property('lazy', relationship(foos, bar_foo, lazy='select'))
+        bars.add_property('eager', relationship(foos, bar_foo, lazy='joined'))
 
         foo.insert().execute(data='foo1')
         bar.insert().execute(id=1, data='bar1')
@@ -344,7 +404,8 @@ class EagerLazyTest(_base.MappedTest):
         self.assert_(len(q.first().eager) == 1)
 
 class EagerTargetingTest(_base.MappedTest):
-    """test a scenario where joined table inheritance might be confused as an eagerly loaded joined table."""
+    """test a scenario where joined table inheritance might be 
+    confused as an eagerly loaded joined table."""
     
     @classmethod
     def define_tables(cls, metadata):
@@ -369,7 +430,7 @@ class EagerTargetingTest(_base.MappedTest):
         
         mapper(A, a_table, polymorphic_on=a_table.c.type, polymorphic_identity='A', 
                 properties={
-                    'children': relation(A, order_by=a_table.c.name)
+                    'children': relationship(A, order_by=a_table.c.name)
             })
 
         mapper(B, b_table, inherits=A, polymorphic_identity='B', properties={
@@ -394,44 +455,44 @@ class EagerTargetingTest(_base.MappedTest):
         eq_(node.children[0], B(id=2, name='b2',b_data='l'))
         
         sess.expunge_all()
-        node = sess.query(B).options(eagerload(B.children)).filter(B.id==bid).all()[0]
+        node = sess.query(B).options(joinedload(B.children)).filter(B.id==bid).all()[0]
         eq_(node, B(id=1, name='b1',b_data='i'))
         eq_(node.children[0], B(id=2, name='b2',b_data='l'))
         
-        
 class FlushTest(_base.MappedTest):
     """test dependency sorting among inheriting mappers"""
+    
     @classmethod
     def define_tables(cls, metadata):
-        global users, roles, user_roles, admins
-        users = Table('users', metadata,
-            Column('id', Integer, primary_key=True),
+        Table('users', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('email', String(128)),
             Column('password', String(16)),
         )
 
-        roles = Table('role', metadata,
-            Column('id', Integer, primary_key=True),
+        Table('roles', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('description', String(32))
         )
 
-        user_roles = Table('user_role', metadata,
+        Table('user_roles', metadata,
             Column('user_id', Integer, ForeignKey('users.id'), primary_key=True),
-            Column('role_id', Integer, ForeignKey('role.id'), primary_key=True)
+            Column('role_id', Integer, ForeignKey('roles.id'), primary_key=True)
         )
 
-        admins = Table('admin', metadata,
-            Column('admin_id', Integer, primary_key=True),
+        Table('admins', metadata,
+            Column('admin_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('user_id', Integer, ForeignKey('users.id'))
         )
 
-    def testone(self):
+    @testing.resolve_artifact_names
+    def test_one(self):
         class User(object):pass
         class Role(object):pass
         class Admin(User):pass
         role_mapper = mapper(Role, roles)
         user_mapper = mapper(User, users, properties = {
-                'roles' : relation(Role, secondary=user_roles, lazy=False)
+                'roles' : relationship(Role, secondary=user_roles, lazy='joined')
             }
         )
         admin_mapper = mapper(Admin, admins, inherits=user_mapper)
@@ -452,7 +513,8 @@ class FlushTest(_base.MappedTest):
 
         assert user_roles.count().scalar() == 1
 
-    def testtwo(self):
+    @testing.resolve_artifact_names
+    def test_two(self):
         class User(object):
             def __init__(self, email=None, password=None):
                 self.email = email
@@ -466,7 +528,7 @@ class FlushTest(_base.MappedTest):
 
         role_mapper = mapper(Role, roles)
         user_mapper = mapper(User, users, properties = {
-                'roles' : relation(Role, secondary=user_roles, lazy=False)
+                'roles' : relationship(Role, secondary=user_roles, lazy='joined')
             }
         )
 
@@ -492,24 +554,24 @@ class FlushTest(_base.MappedTest):
 class VersioningTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        global base, subtable, stuff
-        base = Table('base', metadata,
-            Column('id', Integer, Sequence('version_test_seq', optional=True), primary_key=True ),
+        Table('base', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('version_id', Integer, nullable=False),
             Column('value', String(40)),
             Column('discriminator', Integer, nullable=False)
         )
-        subtable = Table('subtable', metadata,
+        Table('subtable', metadata,
             Column('id', None, ForeignKey('base.id'), primary_key=True),
             Column('subdata', String(50))
             )
-        stuff = Table('stuff', metadata,
-            Column('id', Integer, primary_key=True),
+        Table('stuff', metadata,
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('parent', Integer, ForeignKey('base.id'))
             )
 
-    @testing.fails_on('mssql', 'FIXME: the flush still happens with the concurrency issue.')
+    @testing.emits_warning(r".*updated rowcount")
     @engines.close_open_connections
+    @testing.resolve_artifact_names
     def test_save_update(self):
         class Base(_fixtures.Base):
             pass
@@ -518,8 +580,11 @@ class VersioningTest(_base.MappedTest):
         class Stuff(Base):
             pass
         mapper(Stuff, stuff)
-        mapper(Base, base, polymorphic_on=base.c.discriminator, version_id_col=base.c.version_id, polymorphic_identity=1, properties={
-            'stuff':relation(Stuff)
+        mapper(Base, base, 
+                    polymorphic_on=base.c.discriminator, 
+                    version_id_col=base.c.version_id, 
+                    polymorphic_identity=1, properties={
+            'stuff':relationship(Stuff)
         })
         mapper(Sub, subtable, inherits=Base, polymorphic_identity=2)
 
@@ -540,31 +605,32 @@ class VersioningTest(_base.MappedTest):
 
         sess.flush()
 
-        try:
-            sess2.query(Base).with_lockmode('read').get(s1.id)
-            assert False
-        except orm_exc.ConcurrentModificationError, e:
-            assert True
+        assert_raises(orm_exc.ConcurrentModificationError,
+                        sess2.query(Base).with_lockmode('read').get, 
+                        s1.id)
 
-        try:
+        if not testing.db.dialect.supports_sane_rowcount:
             sess2.flush()
-            assert False
-        except orm_exc.ConcurrentModificationError, e:
-            assert True
+        else:
+            assert_raises(orm_exc.ConcurrentModificationError, sess2.flush)
 
         sess2.refresh(s2)
-        assert s2.subdata == 'sess1 subdata'
+        if testing.db.dialect.supports_sane_rowcount:
+            assert s2.subdata == 'sess1 subdata'
         s2.subdata = 'sess2 subdata'
         sess2.flush()
 
-    @testing.fails_on('mssql', 'FIXME: the flush still happens with the concurrency issue.')
+    @testing.emits_warning(r".*updated rowcount")
+    @testing.resolve_artifact_names
     def test_delete(self):
         class Base(_fixtures.Base):
             pass
         class Sub(Base):
             pass
 
-        mapper(Base, base, polymorphic_on=base.c.discriminator, version_id_col=base.c.version_id, polymorphic_identity=1)
+        mapper(Base, base, 
+                    polymorphic_on=base.c.discriminator, 
+                    version_id_col=base.c.version_id, polymorphic_identity=1)
         mapper(Sub, subtable, inherits=Base, polymorphic_identity=2)
 
         sess = create_session()
@@ -589,7 +655,7 @@ class VersioningTest(_base.MappedTest):
         try:
             s1.subdata = 'some new subdata'
             sess.flush()
-            assert False
+            assert not testing.db.dialect.supports_sane_rowcount
         except orm_exc.ConcurrentModificationError, e:
             assert True
 
@@ -605,12 +671,12 @@ class DistinctPKTest(_base.MappedTest):
         global person_table, employee_table, Person, Employee
 
         person_table = Table("persons", metadata,
-                Column("id", Integer, primary_key=True),
+                Column("id", Integer, primary_key=True, test_needs_autoincrement=True),
                 Column("name", String(80)),
                 )
 
         employee_table = Table("employees", metadata,
-                Column("id", Integer, primary_key=True),
+                Column("id", Integer, primary_key=True, test_needs_autoincrement=True),
                 Column("salary", Integer),
                 Column("person_id", Integer, ForeignKey("persons.id")),
                 )
@@ -638,17 +704,24 @@ class DistinctPKTest(_base.MappedTest):
 
     def test_explicit_props(self):
         person_mapper = mapper(Person, person_table)
-        mapper(Employee, employee_table, inherits=person_mapper, properties={'pid':person_table.c.id, 'eid':employee_table.c.id})
+        mapper(Employee, employee_table, inherits=person_mapper,
+                        properties={'pid':person_table.c.id, 
+                                    'eid':employee_table.c.id})
         self._do_test(True)
 
     def test_explicit_composite_pk(self):
         person_mapper = mapper(Person, person_table)
-        try:
-            mapper(Employee, employee_table, inherits=person_mapper, primary_key=[person_table.c.id, employee_table.c.id])
-            self._do_test(True)
-            assert False
-        except sa_exc.SAWarning, e:
-            assert str(e) == "On mapper Mapper|Employee|employees, primary key column 'employees.id' is being combined with distinct primary key column 'persons.id' in attribute 'id'.  Use explicit properties to give each column its own mapped attribute name.", str(e)
+        mapper(Employee, employee_table, 
+                    inherits=person_mapper, 
+                    primary_key=[person_table.c.id, employee_table.c.id])
+        assert_raises_message(sa_exc.SAWarning, 
+                                    r"On mapper Mapper\|Employee\|employees, "
+                                    "primary key column 'employees.id' is being "
+                                    "combined with distinct primary key column 'persons.id' "
+                                    "in attribute 'id'.  Use explicit properties to give "
+                                    "each column its own mapped attribute name.",
+            self._do_test, True
+        )
 
     def test_explicit_pk(self):
         person_mapper = mapper(Person, person_table)
@@ -673,12 +746,13 @@ class DistinctPKTest(_base.MappedTest):
 
 class SyncCompileTest(_base.MappedTest):
     """test that syncrules compile properly on custom inherit conds"""
+    
     @classmethod
     def define_tables(cls, metadata):
         global _a_table, _b_table, _c_table
 
         _a_table = Table('a', metadata,
-           Column('id', Integer, primary_key=True),
+           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('data1', String(128))
         )
 
@@ -695,7 +769,8 @@ class SyncCompileTest(_base.MappedTest):
 
     def test_joins(self):
         for j1 in (None, _b_table.c.a_id==_a_table.c.id, _a_table.c.id==_b_table.c.a_id):
-            for j2 in (None, _b_table.c.a_id==_c_table.c.b_a_id, _c_table.c.b_a_id==_b_table.c.a_id):
+            for j2 in (None, _b_table.c.a_id==_c_table.c.b_a_id,
+                                    _c_table.c.b_a_id==_b_table.c.a_id):
                 self._do_test(j1, j2)
                 for t in reversed(_a_table.metadata.sorted_tables):
                     t.delete().execute().close()
@@ -746,7 +821,7 @@ class OverrideColKeyTest(_base.MappedTest):
         global base, subtable
         
         base = Table('base', metadata, 
-            Column('base_id', Integer, primary_key=True),
+            Column('base_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(255)),
             Column('sqlite_fixer', String(10))
             )
@@ -865,8 +940,8 @@ class OverrideColKeyTest(_base.MappedTest):
         # gets joined normally with the extra column.
         
         eq_(
-            class_mapper(Sub).get_property('id').columns,
-            [base.c.base_id, subtable.c.base_id]
+            set(class_mapper(Sub).get_property('id').columns),
+            set([base.c.base_id, subtable.c.base_id])
         )
         
         s1 = Sub()
@@ -976,7 +1051,7 @@ class OptimizedLoadTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global base, sub, with_comp
         base = Table('base', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(50)),
             Column('type', String(50))
         )
@@ -1097,7 +1172,7 @@ class PKDiscriminatorTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         parents = Table('parents', metadata,
-                           Column('id', Integer, primary_key=True),
+                           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                            Column('name', String(60)))
                            
         children = Table('children', metadata,
@@ -1119,7 +1194,7 @@ class PKDiscriminatorTest(_base.MappedTest):
             pass
             
         mapper(Parent, parents, properties={
-            'children': relation(Child, backref='parent'),
+            'children': relationship(Child, backref='parent'),
         })
         mapper(Child, children, polymorphic_on=children.c.type,
             polymorphic_identity=1)
@@ -1150,14 +1225,14 @@ class DeleteOrphanTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global single, parent
         single = Table('single', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('type', String(50), nullable=False),
             Column('data', String(50)),
             Column('parent_id', Integer, ForeignKey('parent.id'), nullable=False),
             )
             
         parent = Table('parent', metadata,
-                Column('id', Integer, primary_key=True),
+                Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                 Column('data', String(50))
             )
     
@@ -1174,13 +1249,14 @@ class DeleteOrphanTest(_base.MappedTest):
         mapper(Base, single, polymorphic_on=single.c.type, polymorphic_identity='base')
         mapper(SubClass, inherits=Base, polymorphic_identity='sub')
         mapper(Parent, parent, properties={
-            'related':relation(Base, cascade="all, delete-orphan")
+            'related':relationship(Base, cascade="all, delete-orphan")
         })
         
         sess = create_session()
         s1 = SubClass(data='s1')
         sess.add(s1)
         assert_raises_message(orm_exc.FlushError, 
-            "is not attached to any parent 'Parent' instance via that classes' 'related' attribute", sess.flush)
+            r"is not attached to any parent 'Parent' instance via "
+            "that classes' 'related' attribute", sess.flush)
         
     
