@@ -8,6 +8,7 @@ from sqlalchemy.test import testing
 from test.orm import _base
 from sqlalchemy.orm import attributes
 from sqlalchemy.test.testing import eq_
+from sqlalchemy.test.schema import Table, Column
 
 class Employee(object):
     def __init__(self, name):
@@ -48,31 +49,31 @@ class ConcreteTest(_base.MappedTest):
         global managers_table, engineers_table, hackers_table, companies, employees_table
 
         companies = Table('companies', metadata,
-           Column('id', Integer, primary_key=True),
+           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('name', String(50)))
 
         employees_table = Table('employees', metadata,
-            Column('employee_id', Integer, primary_key=True),
+            Column('employee_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('name', String(50)),
             Column('company_id', Integer, ForeignKey('companies.id'))
         )
         
         managers_table = Table('managers', metadata,
-            Column('employee_id', Integer, primary_key=True),
+            Column('employee_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('name', String(50)),
             Column('manager_data', String(50)),
             Column('company_id', Integer, ForeignKey('companies.id'))
         )
 
         engineers_table = Table('engineers', metadata,
-            Column('employee_id', Integer, primary_key=True),
+            Column('employee_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('name', String(50)),
             Column('engineer_info', String(50)),
             Column('company_id', Integer, ForeignKey('companies.id'))
         )
 
         hackers_table = Table('hackers', metadata,
-            Column('employee_id', Integer, primary_key=True),
+            Column('employee_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('name', String(50)),
             Column('engineer_info', String(50)),
             Column('company_id', Integer, ForeignKey('companies.id')),
@@ -285,14 +286,14 @@ class ConcreteTest(_base.MappedTest):
             set(["Engineer Jerry knows how to program", "Hacker Kurt 'Badass' knows how to hack"])
         )
         
-    def test_relation(self):
+    def test_relationship(self):
         pjoin = polymorphic_union({
             'manager':managers_table,
             'engineer':engineers_table
         }, 'type', 'pjoin')
 
         mapper(Company, companies, properties={
-            'employees':relation(Employee)
+            'employees':relationship(Employee)
         })
         employee_mapper = mapper(Employee, pjoin, polymorphic_on=pjoin.c.type)
         manager_mapper = mapper(Manager, managers_table, inherits=employee_mapper, concrete=True, polymorphic_identity='manager')
@@ -312,7 +313,7 @@ class ConcreteTest(_base.MappedTest):
         self.assert_sql_count(testing.db, go, 2)
         session.expunge_all()
         def go():
-            c2 = session.query(Company).options(eagerload(Company.employees)).get(c.id)
+            c2 = session.query(Company).options(joinedload(Company.employees)).get(c.id)
             assert set([repr(x) for x in c2.employees]) == set(["Engineer Kurt knows how to hack", "Manager Tom knows how to manage things"])
         self.assert_sql_count(testing.db, go, 1)
 
@@ -320,17 +321,17 @@ class PropertyInheritanceTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('a_table', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('some_c_id', Integer, ForeignKey('c_table.id')),
             Column('aname', String(50)),
         )
         Table('b_table', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('some_c_id', Integer, ForeignKey('c_table.id')),
             Column('bname', String(50)),
         )
         Table('c_table', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('cname', String(50)),
             
         )
@@ -349,7 +350,7 @@ class PropertyInheritanceTest(_base.MappedTest):
     @testing.resolve_artifact_names    
     def test_noninherited_warning(self):
         mapper(A, a_table, properties={
-            'some_c':relation(C)
+            'some_c':relationship(C)
         })
         mapper(B, b_table,inherits=A, concrete=True)
         mapper(C, c_table)
@@ -377,14 +378,14 @@ class PropertyInheritanceTest(_base.MappedTest):
     @testing.resolve_artifact_names    
     def test_inheriting(self):
         mapper(A, a_table, properties={
-            'some_c':relation(C, back_populates='many_a')
+            'some_c':relationship(C, back_populates='many_a')
         })
         mapper(B, b_table,inherits=A, concrete=True, properties={
-            'some_c':relation(C, back_populates='many_b')
+            'some_c':relationship(C, back_populates='many_b')
         })
         mapper(C, c_table, properties={
-            'many_a':relation(A, back_populates='some_c'),
-            'many_b':relation(B, back_populates='some_c'),
+            'many_a':relationship(A, back_populates='some_c'),
+            'many_b':relationship(B, back_populates='some_c'),
         })
         
         sess = sessionmaker()()
@@ -425,15 +426,15 @@ class PropertyInheritanceTest(_base.MappedTest):
         mapper(A, a_table, with_polymorphic=('*', ajoin), 
             polymorphic_on=ajoin.c.type, polymorphic_identity='a', 
             properties={
-            'some_c':relation(C, back_populates='many_a')
+            'some_c':relationship(C, back_populates='many_a')
         })
         mapper(B, b_table,inherits=A, concrete=True, 
             polymorphic_identity='b', 
             properties={
-            'some_c':relation(C, back_populates='many_a')
+            'some_c':relationship(C, back_populates='many_a')
         })
         mapper(C, c_table, properties={
-            'many_a':relation(A, back_populates='some_c', order_by=ajoin.c.id),
+            'many_a':relationship(A, back_populates='some_c', order_by=ajoin.c.id),
         })
         
         sess = sessionmaker()()
@@ -461,7 +462,7 @@ class PropertyInheritanceTest(_base.MappedTest):
         def go():
             eq_(
                 [C(many_a=[A(aname='a1'), B(bname='b1'), B(bname='b2')]), C(many_a=[A(aname='a2')])],
-                sess.query(C).options(eagerload(C.many_a)).order_by(C.id).all(),
+                sess.query(C).options(joinedload(C.many_a)).order_by(C.id).all(),
             )
         self.assert_sql_count(testing.db, go, 1)
 
@@ -497,12 +498,12 @@ class ManyToManyTest(_base.MappedTest):
             pass
 
     @testing.resolve_artifact_names
-    def test_selective_relations(self):
+    def test_selective_relationships(self):
         mapper(Base, base, properties={
-            'related':relation(Related, secondary=base_mtom, backref='bases', order_by=related.c.id)
+            'related':relationship(Related, secondary=base_mtom, backref='bases', order_by=related.c.id)
         })
         mapper(Sub, sub, inherits=Base, concrete=True, properties={
-            'related':relation(Related, secondary=sub_mtom, backref='subs', order_by=related.c.id)
+            'related':relationship(Related, secondary=sub_mtom, backref='subs', order_by=related.c.id)
         })
         mapper(Related, related)
         
@@ -525,11 +526,11 @@ class ColKeysTest(_base.MappedTest):
     def define_tables(cls, metadata):
         global offices_table, refugees_table
         refugees_table = Table('refugee', metadata,
-           Column('refugee_fid', Integer, primary_key=True),
+           Column('refugee_fid', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('refugee_name', Unicode(30), key='name'))
 
         offices_table = Table('office', metadata,
-           Column('office_fid', Integer, primary_key=True),
+           Column('office_fid', Integer, primary_key=True, test_needs_autoincrement=True),
            Column('office_name', Unicode(30), key='name'))
     
     @classmethod
