@@ -8,12 +8,11 @@ from sqlalchemy.orm.collections import collection
 import sqlalchemy as sa
 from sqlalchemy.test import testing
 from sqlalchemy import Integer, String, ForeignKey
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from sqlalchemy.test.schema import Table, Column
 from sqlalchemy import util, exc as sa_exc
-from sqlalchemy.orm import create_session, mapper, relation,     attributes
+from sqlalchemy.orm import create_session, mapper, relationship, attributes
 from test.orm import _base
-from sqlalchemy.test.testing import eq_
+from sqlalchemy.test.testing import eq_, assert_raises
 
 class Canary(sa.orm.interfaces.AttributeExtension):
     def __init__(self):
@@ -169,6 +168,13 @@ class CollectionsTest(_base.ORMTest):
                 control[slice(0,-1)] = values
                 assert_eq()
 
+                values = [creator(),creator(),creator()]
+                control[:] = values
+                direct[:] = values
+                def invalid():
+                    direct[slice(0, 6, 2)] = [creator()]
+                assert_raises(ValueError, invalid)
+                
         if hasattr(direct, '__delitem__'):
             e = creator()
             direct.append(e)
@@ -193,7 +199,7 @@ class CollectionsTest(_base.ORMTest):
                 del direct[::2]
                 del control[::2]
                 assert_eq()
-
+            
         if hasattr(direct, 'remove'):
             e = creator()
             direct.append(e)
@@ -202,8 +208,21 @@ class CollectionsTest(_base.ORMTest):
             direct.remove(e)
             control.remove(e)
             assert_eq()
-
-        if hasattr(direct, '__setslice__'):
+        
+        if hasattr(direct, '__setitem__') or hasattr(direct, '__setslice__'):
+            
+            values = [creator(), creator()]
+            direct[:] = values
+            control[:] = values
+            assert_eq()
+            
+            # test slice assignment where
+            # slice size goes over the number of items
+            values = [creator(), creator()]
+            direct[1:3] = values
+            control[1:3] = values
+            assert_eq()
+            
             values = [creator(), creator()]
             direct[0:1] = values
             control[0:1] = values
@@ -228,8 +247,19 @@ class CollectionsTest(_base.ORMTest):
             direct[1::2] = values
             control[1::2] = values
             assert_eq()
+            
+            values = [creator(), creator()]
+            direct[-1:-3] = values
+            control[-1:-3] = values
+            assert_eq()
 
-        if hasattr(direct, '__delslice__'):
+            values = [creator(), creator()]
+            direct[-2:-1] = values
+            control[-2:-1] = values
+            assert_eq()
+            
+
+        if hasattr(direct, '__delitem__') or hasattr(direct, '__delslice__'):
             for i in range(1, 4):
                 e = creator()
                 direct.append(e)
@@ -246,7 +276,7 @@ class CollectionsTest(_base.ORMTest):
             del direct[:]
             del control[:]
             assert_eq()
-
+        
         if hasattr(direct, 'extend'):
             values = [creator(), creator(), creator()]
 
@@ -345,6 +375,45 @@ class CollectionsTest(_base.ORMTest):
         self._test_list(list)
         self._test_list_bulk(list)
 
+    def test_list_setitem_with_slices(self):
+        
+        # this is a "list" that has no __setslice__
+        # or __delslice__ methods.  The __setitem__
+        # and __delitem__ must therefore accept
+        # slice objects (i.e. as in py3k)
+        class ListLike(object):
+            def __init__(self):
+                self.data = list()
+            def append(self, item):
+                self.data.append(item)
+            def remove(self, item):
+                self.data.remove(item)
+            def insert(self, index, item):
+                self.data.insert(index, item)
+            def pop(self, index=-1):
+                return self.data.pop(index)
+            def extend(self):
+                assert False
+            def __len__(self):
+                return len(self.data)
+            def __setitem__(self, key, value):
+                self.data[key] = value
+            def __getitem__(self, key):
+                return self.data[key]
+            def __delitem__(self, key):
+                del self.data[key]
+            def __iter__(self):
+                return iter(self.data)
+            __hash__ = object.__hash__
+            def __eq__(self, other):
+                return self.data == other
+            def __repr__(self):
+                return 'ListLike(%s)' % repr(self.data)
+
+        self._test_adapter(ListLike)
+        self._test_list(ListLike)
+        self._test_list_bulk(ListLike)
+
     def test_list_subclass(self):
         class MyList(list):
             pass
@@ -1343,10 +1412,10 @@ class DictHelpersTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('parents', metadata,
-              Column('id', Integer, primary_key=True),
+              Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
               Column('label', String(128)))
         Table('children', metadata,
-              Column('id', Integer, primary_key=True),
+              Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
               Column('parent_id', Integer, ForeignKey('parents.id'),
                      nullable=False),
               Column('a', String(128)),
@@ -1369,7 +1438,7 @@ class DictHelpersTest(_base.MappedTest):
     def _test_scalar_mapped(self, collection_class):
         mapper(Child, children)
         mapper(Parent, parents, properties={
-            'children': relation(Child, collection_class=collection_class,
+            'children': relationship(Child, collection_class=collection_class,
                                  cascade="all, delete-orphan")})
 
         p = Parent()
@@ -1428,7 +1497,7 @@ class DictHelpersTest(_base.MappedTest):
     def _test_composite_mapped(self, collection_class):
         mapper(Child, children)
         mapper(Parent, parents, properties={
-            'children': relation(Child, collection_class=collection_class,
+            'children': relationship(Child, collection_class=collection_class,
                                  cascade="all, delete-orphan")
             })
 
@@ -1481,14 +1550,14 @@ class DictHelpersTest(_base.MappedTest):
 
         class Foo(BaseObject):
             __tablename__ = "foo"
-            id = Column(Integer(), primary_key=True)
+            id = Column(Integer(), primary_key=True, test_needs_autoincrement=True)
             bar_id = Column(Integer, ForeignKey('bar.id'))
             
         class Bar(BaseObject):
             __tablename__ = "bar"
-            id = Column(Integer(), primary_key=True)
-            foos = relation(Foo, collection_class=collections.column_mapped_collection(Foo.id))
-            foos2 = relation(Foo, collection_class=collections.column_mapped_collection((Foo.id, Foo.bar_id)))
+            id = Column(Integer(), primary_key=True, test_needs_autoincrement=True)
+            foos = relationship(Foo, collection_class=collections.column_mapped_collection(Foo.id))
+            foos2 = relationship(Foo, collection_class=collections.column_mapped_collection((Foo.id, Foo.bar_id)))
             
         eq_(Bar.foos.property.collection_class().keyfunc(Foo(id=3)), 3)
         eq_(Bar.foos2.property.collection_class().keyfunc(Foo(id=3, bar_id=12)), (3, 12))
@@ -1521,17 +1590,16 @@ class DictHelpersTest(_base.MappedTest):
         collection_class = lambda: Ordered2(lambda v: (v.a, v.b))
         self._test_composite_mapped(collection_class)
 
-# TODO: are these tests redundant vs. the above tests ?
-# remove if so
 class CustomCollectionsTest(_base.MappedTest):
+    """test the integration of collections with mapped classes."""
 
     @classmethod
     def define_tables(cls, metadata):
         Table('sometable', metadata,
-              Column('col1',Integer, primary_key=True),
+              Column('col1',Integer, primary_key=True, test_needs_autoincrement=True),
               Column('data', String(30)))
         Table('someothertable', metadata,
-              Column('col1', Integer, primary_key=True),
+              Column('col1', Integer, primary_key=True, test_needs_autoincrement=True),
               Column('scol1', Integer,
                      ForeignKey('sometable.col1')),
               Column('data', String(20)))
@@ -1546,7 +1614,7 @@ class CustomCollectionsTest(_base.MappedTest):
             pass
 
         mapper(Foo, sometable, properties={
-            'bars':relation(Bar, collection_class=MyList)
+            'bars':relationship(Bar, collection_class=MyList)
         })
         mapper(Bar, someothertable)
         f = Foo()
@@ -1560,7 +1628,7 @@ class CustomCollectionsTest(_base.MappedTest):
         class Bar(object):
             pass
         mapper(Foo, sometable, properties={
-            'bars':relation(Bar, collection_class=set)
+            'bars':relationship(Bar, collection_class=set)
         })
         mapper(Bar, someothertable)
         f = Foo()
@@ -1592,7 +1660,7 @@ class CustomCollectionsTest(_base.MappedTest):
                     del self[id(item)]
 
         mapper(Foo, sometable, properties={
-            'bars':relation(Bar, collection_class=AppenderDict)
+            'bars':relationship(Bar, collection_class=AppenderDict)
         })
         mapper(Bar, someothertable)
         f = Foo()
@@ -1616,7 +1684,7 @@ class CustomCollectionsTest(_base.MappedTest):
             def __init__(self, data): self.data = data
 
         mapper(Foo, sometable, properties={
-            'bars':relation(Bar,
+            'bars':relationship(Bar,
                 collection_class=collections.column_mapped_collection(
                     someothertable.c.data))
         })
@@ -1646,15 +1714,50 @@ class CustomCollectionsTest(_base.MappedTest):
         replaced = set([id(b) for b in f.bars.values()])
         self.assert_(existing != replaced)
 
-    @testing.resolve_artifact_names
     def test_list(self):
+        self._test_list(list)
+
+    def test_list_no_setslice(self):
+        class ListLike(object):
+            def __init__(self):
+                self.data = list()
+            def append(self, item):
+                self.data.append(item)
+            def remove(self, item):
+                self.data.remove(item)
+            def insert(self, index, item):
+                self.data.insert(index, item)
+            def pop(self, index=-1):
+                return self.data.pop(index)
+            def extend(self):
+                assert False
+            def __len__(self):
+                return len(self.data)
+            def __setitem__(self, key, value):
+                self.data[key] = value
+            def __getitem__(self, key):
+                return self.data[key]
+            def __delitem__(self, key):
+                del self.data[key]
+            def __iter__(self):
+                return iter(self.data)
+            __hash__ = object.__hash__
+            def __eq__(self, other):
+                return self.data == other
+            def __repr__(self):
+                return 'ListLike(%s)' % repr(self.data)
+        
+        self._test_list(ListLike)
+        
+    @testing.resolve_artifact_names
+    def _test_list(self, listcls):
         class Parent(object):
             pass
         class Child(object):
             pass
 
         mapper(Parent, sometable, properties={
-            'children':relation(Child, collection_class=list)
+            'children':relationship(Child, collection_class=listcls)
         })
         mapper(Child, someothertable)
 
@@ -1789,7 +1892,7 @@ class CustomCollectionsTest(_base.MappedTest):
                 return iter(self.data)
 
         mapper(Parent, sometable, properties={
-            'children':relation(Child, collection_class=MyCollection)
+            'children':relationship(Child, collection_class=MyCollection)
         })
         mapper(Child, someothertable)
 
