@@ -1,15 +1,16 @@
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
-import gc
+from sqlalchemy.test.testing import eq_, assert_raises, \
+    assert_raises_message
+from sqlalchemy.test.util import gc_collect
 import inspect
 import pickle
-from sqlalchemy.orm import create_session, sessionmaker, attributes
+from sqlalchemy.orm import create_session, sessionmaker, attributes, \
+    make_transient
 from sqlalchemy.orm.attributes import instance_state
 import sqlalchemy as sa
 from sqlalchemy.test import engines, testing, config
 from sqlalchemy import Integer, String, Sequence
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
-from sqlalchemy.orm import mapper, relation, backref, eagerload
+from sqlalchemy.test.schema import Table, Column
+from sqlalchemy.orm import mapper, relationship, backref, joinedload
 from sqlalchemy.test.testing import eq_
 from test.engine import _base as engine_base
 from test.orm import _base, _fixtures
@@ -81,7 +82,7 @@ class SessionTest(_fixtures.FixtureTest):
     def test_expunge_cascade(self):
         mapper(Address, addresses)
         mapper(User, users, properties={
-            'addresses':relation(Address,
+            'addresses':relationship(Address,
                                  backref=backref("user", cascade="all"),
                                  cascade="all")})
 
@@ -114,7 +115,7 @@ class SessionTest(_fixtures.FixtureTest):
 
         mapper(Address, addresses_unbound)
         mapper(User, users_unbound, properties={
-            'addresses':relation(Address,
+            'addresses':relationship(Address,
                                  backref=backref("user", cascade="all"),
                                  cascade="all")})
 
@@ -128,9 +129,11 @@ class SessionTest(_fixtures.FixtureTest):
             [User(id=1, name='ed')])
 
         # test expression binding
-        sess.execute(users_unbound.insert(), params=dict(id=2, name='jack'))
-        eq_(sess.execute(users_unbound.select(users_unbound.c.id == 2)).fetchall(),
-            [(2, 'jack')])
+
+        sess.execute(users_unbound.insert(), params=dict(id=2,
+                     name='jack'))
+        eq_(sess.execute(users_unbound.select(users_unbound.c.id
+            == 2)).fetchall(), [(2, 'jack')])
 
         eq_(sess.execute(users_unbound.select(User.id == 2)).fetchall(),
             [(2, 'jack')])
@@ -151,7 +154,7 @@ class SessionTest(_fixtures.FixtureTest):
 
         mapper(Address, addresses_unbound)
         mapper(User, users_unbound, properties={
-            'addresses':relation(Address,
+            'addresses':relationship(Address,
                                  backref=backref("user", cascade="all"),
                                  cascade="all")})
 
@@ -165,8 +168,9 @@ class SessionTest(_fixtures.FixtureTest):
             [User(id=1, name='ed')])
 
         sess.execute(users_unbound.insert(), params=dict(id=2, name='jack'))
-        eq_(sess.execute(users_unbound.select(users_unbound.c.id == 2)).fetchall(),
-            [(2, 'jack')])
+
+        eq_(sess.execute(users_unbound.select(users_unbound.c.id
+            == 2)).fetchall(), [(2, 'jack')])
 
         eq_(sess.execute(users_unbound.select(User.id == 2)).fetchall(),
             [(2, 'jack')])
@@ -207,7 +211,9 @@ class SessionTest(_fixtures.FixtureTest):
         assert conn2.execute("select count(1) from users").scalar() == 0
         sess.commit()
         assert conn1.execute("select count(1) from users").scalar() == 1
-        assert testing.db.connect().execute("select count(1) from users").scalar() == 1
+
+        assert testing.db.connect().execute('select count(1) from users'
+                ).scalar() == 1
         sess.close()
 
     @testing.requires.independent_connections
@@ -232,6 +238,34 @@ class SessionTest(_fixtures.FixtureTest):
         eq_(bind.connect().execute("select count(1) from users").scalar(), 1)
         sess.close()
 
+    @testing.resolve_artifact_names
+    def test_make_transient(self):
+        mapper(User, users)
+        sess = create_session()
+        sess.add(User(name='test'))
+        sess.flush()
+        
+        u1 = sess.query(User).first()
+        make_transient(u1)
+        assert u1 not in sess
+        sess.add(u1)
+        assert u1 in sess.new
+
+        u1 = sess.query(User).first()
+        sess.expunge(u1)
+        make_transient(u1)
+        sess.add(u1)
+        assert u1 in sess.new
+        
+        # test expired attributes 
+        # get unexpired
+        u1 = sess.query(User).first()
+        sess.expire(u1)
+        make_transient(u1)
+        assert u1.id is None
+        assert u1.name is None
+
+        
     @testing.resolve_artifact_names
     def test_autoflush_expressions(self):
         """test that an expression which is dependent on object state is 
@@ -240,7 +274,7 @@ class SessionTest(_fixtures.FixtureTest):
         
         """
         mapper(User, users, properties={
-            'addresses':relation(Address, backref="user")})
+            'addresses':relationship(Address, backref="user")})
         mapper(Address, addresses)
 
         sess = create_session(autoflush=True, autocommit=False)
@@ -255,28 +289,32 @@ class SessionTest(_fixtures.FixtureTest):
         u = sess.query(User).get(u.id)
         q = sess.query(Address).filter(Address.user==u)
         del u
-        gc.collect()
+        gc_collect()
         eq_(q.one(), Address(email_address='foo'))
 
 
+
     @testing.requires.independent_connections
     @engines.close_open_connections
     @testing.resolve_artifact_names
     def test_autoflush_unbound(self):
         mapper(User, users)
-
         try:
             sess = create_session(autocommit=False, autoflush=True)
             u = User()
-            u.name='ed'
+            u.name = 'ed'
             sess.add(u)
             u2 = sess.query(User).filter_by(name='ed').one()
             assert u2 is u
-            assert sess.execute("select count(1) from users", mapper=User).scalar() == 1
-            assert testing.db.connect().execute("select count(1) from users").scalar() == 0
+            assert sess.execute('select count(1) from users',
+                                mapper=User).scalar() == 1
+            assert testing.db.connect().execute('select count(1) from '
+                    'users').scalar() == 0
             sess.commit()
-            assert sess.execute("select count(1) from users", mapper=User).scalar() == 1
-            assert testing.db.connect().execute("select count(1) from users").scalar() == 1
+            assert sess.execute('select count(1) from users',
+                                mapper=User).scalar() == 1
+            assert testing.db.connect().execute('select count(1) from '
+                    'users').scalar() == 1
             sess.close()
         except:
             sess.rollback()
@@ -288,21 +326,22 @@ class SessionTest(_fixtures.FixtureTest):
         mapper(User, users)
         conn1 = testing.db.connect()
         conn2 = testing.db.connect()
-
-        sess = create_session(bind=conn1, autocommit=False, autoflush=True)
+        sess = create_session(bind=conn1, autocommit=False,
+                              autoflush=True)
         u = User()
-        u.name='ed'
+        u.name = 'ed'
         sess.add(u)
         sess.commit()
-        assert conn1.execute("select count(1) from users").scalar() == 1
-        assert testing.db.connect().execute("select count(1) from users").scalar() == 1
+        assert conn1.execute('select count(1) from users').scalar() == 1
+        assert testing.db.connect().execute('select count(1) from users'
+                ).scalar() == 1
         sess.commit()
 
     @testing.resolve_artifact_names
     def test_autoflush_rollback(self):
         mapper(Address, addresses)
         mapper(User, users, properties={
-            'addresses':relation(Address)})
+            'addresses':relationship(Address)})
 
         _fixtures.run_inserts_for(users)
         _fixtures.run_inserts_for(addresses)
@@ -405,23 +444,23 @@ class SessionTest(_fixtures.FixtureTest):
     @testing.resolve_artifact_names
     def test_heavy_nesting(self):
         session = create_session(bind=testing.db)
-
         session.begin()
-        session.connection().execute("insert into users (name) values ('user1')")
-
+        session.connection().execute(users.insert().values(name='user1'
+                ))
         session.begin(subtransactions=True)
-
         session.begin_nested()
-
-        session.connection().execute("insert into users (name) values ('user2')")
-        assert session.connection().execute("select count(1) from users").scalar() == 2
-
+        session.connection().execute(users.insert().values(name='user2'
+                ))
+        assert session.connection().execute('select count(1) from users'
+                ).scalar() == 2
         session.rollback()
-        assert session.connection().execute("select count(1) from users").scalar() == 1
-        session.connection().execute("insert into users (name) values ('user3')")
-
+        assert session.connection().execute('select count(1) from users'
+                ).scalar() == 1
+        session.connection().execute(users.insert().values(name='user3'
+                ))
         session.commit()
-        assert session.connection().execute("select count(1) from users").scalar() == 2
+        assert session.connection().execute('select count(1) from users'
+                ).scalar() == 2
 
     @testing.fails_on('sqlite', 'FIXME: unknown')
     @testing.resolve_artifact_names
@@ -606,27 +645,26 @@ class SessionTest(_fixtures.FixtureTest):
     @testing.resolve_artifact_names
     def test_error_on_using_inactive_session(self):
         mapper(User, users)
-
         sess = create_session(autocommit=True)
-
         sess.begin()
         sess.begin(subtransactions=True)
-
         sess.add(User(name='u1'))
         sess.flush()
-
         sess.rollback()
-        assert_raises_message(sa.exc.InvalidRequestError, "inactive due to a rollback in a subtransaction", sess.begin, subtransactions=True)
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'inactive due to a rollback in a '
+                              'subtransaction', sess.begin,
+                              subtransactions=True)
         sess.close()
 
     @testing.resolve_artifact_names
     def test_no_autocommit_with_explicit_commit(self):
         mapper(User, users)
         session = create_session(autocommit=False)
-
         session.add(User(name='ed'))
         session.transaction.commit()
-        assert session.transaction is not None, "autocommit=False should start a new transaction"
+        assert session.transaction is not None, \
+            'autocommit=False should start a new transaction'
 
     @engines.close_open_connections
     @testing.resolve_artifact_names
@@ -639,10 +677,14 @@ class SessionTest(_fixtures.FixtureTest):
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        assert transaction._connection_for_bind(testing.db) is transaction._connection_for_bind(c) is c
-
-        assert_raises_message(sa.exc.InvalidRequestError, "Session already has a Connection associated", transaction._connection_for_bind, testing.db.connect())
+        assert transaction._connection_for_bind(testing.db) \
+            is transaction._connection_for_bind(c) is c
 
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'Session already has a Connection '
+                              'associated',
+                              transaction._connection_for_bind,
+                              testing.db.connect())
         transaction.rollback()
         assert len(sess.query(User).all()) == 0
         sess.close()
@@ -683,21 +725,20 @@ class SessionTest(_fixtures.FixtureTest):
         assert c.scalar("select count(1) from users") == 1
 
 
-    @testing.uses_deprecated()
     @engines.close_open_connections
     @testing.resolve_artifact_names
-    def test_save_update_delete(self):
+    def test_add_delete(self):
 
         s = create_session()
         mapper(User, users, properties={
-            'addresses':relation(Address, cascade="all, delete")
+            'addresses':relationship(Address, cascade="all, delete")
         })
         mapper(Address, addresses)
 
         user = User(name='u1')
 
-        assert_raises_message(sa.exc.InvalidRequestError, "is not persisted", s.update, user)
-        assert_raises_message(sa.exc.InvalidRequestError, "is not persisted", s.delete, user)
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'is not persisted', s.delete, user)
 
         s.add(user)
         s.flush()
@@ -723,14 +764,13 @@ class SessionTest(_fixtures.FixtureTest):
         assert user in s
         assert user not in s.dirty
 
-        assert_raises_message(sa.exc.InvalidRequestError, "is already persistent", s.save, user)
-
         s2 = create_session()
-        assert_raises_message(sa.exc.InvalidRequestError, "is already attached to session", s2.delete, user)
-
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'is already attached to session',
+                              s2.delete, user)
         u2 = s2.query(User).get(user.id)
-        assert_raises_message(sa.exc.InvalidRequestError, "another instance with key", s.delete, u2)
-
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'another instance with key', s.delete, u2)
         s.expire(user)
         s.expunge(user)
         assert user not in s
@@ -745,7 +785,7 @@ class SessionTest(_fixtures.FixtureTest):
     def test_is_modified(self):
         s = create_session()
 
-        mapper(User, users, properties={'addresses':relation(Address)})
+        mapper(User, users, properties={'addresses':relationship(Address)})
         mapper(Address, addresses)
 
         # save user
@@ -786,7 +826,8 @@ class SessionTest(_fixtures.FixtureTest):
 
     @testing.resolve_artifact_names
     def test_weak_ref(self):
-        """test the weak-referencing identity map, which strongly-references modified items."""
+        """test the weak-referencing identity map, which strongly-
+        references modified items."""
 
         s = create_session()
         mapper(User, users)
@@ -797,18 +838,18 @@ class SessionTest(_fixtures.FixtureTest):
 
         user = s.query(User).one()
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 0
 
         user = s.query(User).one()
         user.name = 'fred'
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 1
         assert len(s.dirty) == 1
         assert None not in s.dirty
         s.flush()
-        gc.collect()
+        gc_collect()
         assert not s.dirty
         assert not s.identity_map
 
@@ -835,13 +876,13 @@ class SessionTest(_fixtures.FixtureTest):
         s.add(u2)
         
         del u2
-        gc.collect()
+        gc_collect()
         
         assert len(s.identity_map) == 1
         assert len(s.dirty) == 1
         assert None not in s.dirty
         s.flush()
-        gc.collect()
+        gc_collect()
         assert not s.dirty
         
         assert not s.identity_map
@@ -862,66 +903,67 @@ class SessionTest(_fixtures.FixtureTest):
             s.expunge(u2)
             s.identity_map.add(sa.orm.attributes.instance_state(u1))
 
-            assert_raises(AssertionError, s.identity_map.add, sa.orm.attributes.instance_state(u2))
+            assert_raises(AssertionError, s.identity_map.add,
+                          sa.orm.attributes.instance_state(u2))
         
         
     @testing.resolve_artifact_names
     def test_weakref_with_cycles_o2m(self):
         s = sessionmaker()()
         mapper(User, users, properties={
-            "addresses":relation(Address, backref="user")
+            "addresses":relationship(Address, backref="user")
         })
         mapper(Address, addresses)
         s.add(User(name="ed", addresses=[Address(email_address="ed1")]))
         s.commit()
         
-        user = s.query(User).options(eagerload(User.addresses)).one()
+        user = s.query(User).options(joinedload(User.addresses)).one()
         user.addresses[0].user # lazyload
         eq_(user, User(name="ed", addresses=[Address(email_address="ed1")]))
         
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 0
 
-        user = s.query(User).options(eagerload(User.addresses)).one()
+        user = s.query(User).options(joinedload(User.addresses)).one()
         user.addresses[0].email_address='ed2'
         user.addresses[0].user # lazyload
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 2
         
         s.commit()
-        user = s.query(User).options(eagerload(User.addresses)).one()
+        user = s.query(User).options(joinedload(User.addresses)).one()
         eq_(user, User(name="ed", addresses=[Address(email_address="ed2")]))
         
     @testing.resolve_artifact_names
     def test_weakref_with_cycles_o2o(self):
         s = sessionmaker()()
         mapper(User, users, properties={
-            "address":relation(Address, backref="user", uselist=False)
+            "address":relationship(Address, backref="user", uselist=False)
         })
         mapper(Address, addresses)
         s.add(User(name="ed", address=Address(email_address="ed1")))
         s.commit()
 
-        user = s.query(User).options(eagerload(User.address)).one()
+        user = s.query(User).options(joinedload(User.address)).one()
         user.address.user
         eq_(user, User(name="ed", address=Address(email_address="ed1")))
 
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 0
 
-        user = s.query(User).options(eagerload(User.address)).one()
+        user = s.query(User).options(joinedload(User.address)).one()
         user.address.email_address='ed2'
         user.address.user # lazyload
 
         del user
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 2
         
         s.commit()
-        user = s.query(User).options(eagerload(User.address)).one()
+        user = s.query(User).options(joinedload(User.address)).one()
         eq_(user, User(name="ed", address=Address(email_address="ed2")))
     
     @testing.resolve_artifact_names
@@ -935,8 +977,7 @@ class SessionTest(_fixtures.FixtureTest):
         user = s.query(User).one()
         user = None
         print s.identity_map
-        import gc
-        gc.collect()
+        gc_collect()
         assert len(s.identity_map) == 1
 
         user = s.query(User).one()
@@ -946,7 +987,7 @@ class SessionTest(_fixtures.FixtureTest):
         s.flush()
         eq_(users.select().execute().fetchall(), [(user.id, 'u2')])
         
-        
+    @testing.fails_on('+zxjdbc', 'http://www.sqlalchemy.org/trac/ticket/1473')
     @testing.resolve_artifact_names
     def test_prune(self):
         s = create_session(weak_identity_map=False)
@@ -959,8 +1000,7 @@ class SessionTest(_fixtures.FixtureTest):
         self.assert_(len(s.identity_map) == 0)
         self.assert_(s.prune() == 0)
         s.flush()
-        import gc
-        gc.collect()
+        gc_collect()
         self.assert_(s.prune() == 9)
         self.assert_(len(s.identity_map) == 1)
 
@@ -1001,7 +1041,7 @@ class SessionTest(_fixtures.FixtureTest):
     def test_no_save_cascade_1(self):
         mapper(Address, addresses)
         mapper(User, users, properties=dict(
-            addresses=relation(Address, cascade="none", backref="user")))
+            addresses=relationship(Address, cascade="none", backref="user")))
         s = create_session()
 
         u = User(name='u1')
@@ -1020,7 +1060,7 @@ class SessionTest(_fixtures.FixtureTest):
     def test_no_save_cascade_2(self):
         mapper(Address, addresses)
         mapper(User, users, properties=dict(
-            addresses=relation(Address,
+            addresses=relationship(Address,
                                cascade="all",
                                backref=backref("user", cascade="none"))))
 
@@ -1057,43 +1097,61 @@ class SessionTest(_fixtures.FixtureTest):
                 log.append('after_begin')
             def after_attach(self, session, instance):
                 log.append('after_attach')
-            def after_bulk_update(self, session, query, query_context, result):
+            def after_bulk_update(
+                self,
+                session,
+                query,
+                query_context,
+                result,
+                ):
                 log.append('after_bulk_update')
-            def after_bulk_delete(self, session, query, query_context, result):
+
+            def after_bulk_delete(
+                self,
+                session,
+                query,
+                query_context,
+                result,
+                ):
                 log.append('after_bulk_delete')
 
         sess = create_session(extension = MyExt())
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        assert log == ['after_attach', 'before_flush', 'after_begin', 'after_flush', 'before_commit', 'after_commit', 'after_flush_postexec']
-
+        assert log == [
+            'after_attach',
+            'before_flush',
+            'after_begin',
+            'after_flush',
+            'before_commit',
+            'after_commit',
+            'after_flush_postexec',
+            ]
         log = []
         sess = create_session(autocommit=False, extension=MyExt())
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        assert log == ['after_attach', 'before_flush', 'after_begin', 'after_flush', 'after_flush_postexec']
-
+        assert log == ['after_attach', 'before_flush', 'after_begin',
+                       'after_flush', 'after_flush_postexec']
         log = []
         u.name = 'ed'
         sess.commit()
-        assert log == ['before_commit', 'before_flush', 'after_flush', 'after_flush_postexec', 'after_commit']
-
+        assert log == ['before_commit', 'before_flush', 'after_flush',
+                       'after_flush_postexec', 'after_commit']
         log = []
         sess.commit()
         assert log == ['before_commit', 'after_commit']
-
         log = []
         sess.query(User).delete()
         assert log == ['after_begin', 'after_bulk_delete']
-
         log = []
         sess.query(User).update({'name': 'foo'})
         assert log == ['after_bulk_update']
-        
         log = []
-        sess = create_session(autocommit=False, extension=MyExt(), bind=testing.db)
+        sess = create_session(autocommit=False, extension=MyExt(),
+                              bind=testing.db)
         conn = sess.connection()
         assert log == ['after_begin']
 
@@ -1110,7 +1168,8 @@ class SessionTest(_fixtures.FixtureTest):
                         session.add(User(name='another %s' % obj.name))
                 for obj in list(session.deleted):
                     if isinstance(obj, User):
-                        x = session.query(User).filter(User.name=='another %s' % obj.name).one()
+                        x = session.query(User).filter(User.name
+                                == 'another %s' % obj.name).one()
                         session.delete(x)
                     
         sess = create_session(extension = MyExt(), autoflush=True)
@@ -1190,21 +1249,20 @@ class SessionTest(_fixtures.FixtureTest):
         
         sess = create_session(extension=MyExt())
         sess.add(User(name='foo'))
-        assert_raises_message(sa.exc.InvalidRequestError, "already flushing", sess.flush)
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'already flushing', sess.flush)
 
     @testing.resolve_artifact_names
     def test_pickled_update(self):
         mapper(User, users)
         sess1 = create_session()
         sess2 = create_session()
-
         u1 = User(name='u1')
         sess1.add(u1)
-
-        assert_raises_message(sa.exc.InvalidRequestError, "already attached to session", sess2.add, u1)
-
+        assert_raises_message(sa.exc.InvalidRequestError,
+                              'already attached to session', sess2.add,
+                              u1)
         u2 = pickle.loads(pickle.dumps(u1))
-
         sess2.add(u2)
 
     @testing.resolve_artifact_names
@@ -1272,10 +1330,9 @@ class DisposedStates(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         global t1
-        t1 = Table('t1', metadata, 
-            Column('id', Integer, primary_key=True),
-            Column('data', String(50))
-            )
+        t1 = Table('t1', metadata, Column('id', Integer,
+                   primary_key=True, test_needs_autoincrement=True),
+                   Column('data', String(50)))
 
     @classmethod
     def setup_mappers(cls):
@@ -1291,20 +1348,22 @@ class DisposedStates(_base.MappedTest):
         super(DisposedStates, self).teardown()
         
     def _set_imap_in_disposal(self, sess, *objs):
-        """remove selected objects from the given session, as though they 
-        were dereferenced and removed from WeakIdentityMap.
+        """remove selected objects from the given session, as though
+        they were dereferenced and removed from WeakIdentityMap.
         
-        Hardcodes the identity map's "all_states()" method to return the full list
-        of states.  This simulates the all_states() method returning results, afterwhich
-        some of the states get garbage collected (this normally only happens during
-        asynchronous gc).  The Session now has one or more 
-        InstanceState's which have been removed from the identity map and disposed.
+        Hardcodes the identity map's "all_states()" method to return the
+        full list of states.  This simulates the all_states() method
+        returning results, afterwhich some of the states get garbage
+        collected (this normally only happens during asynchronous gc).
+        The Session now has one or more InstanceState's which have been
+        removed from the identity map and disposed.
         
         Will the Session not trip over this ???  Stay tuned.
         
         """
+
         all_states = sess.identity_map.all_states()
-        sess.identity_map.all_states = lambda: all_states
+        sess.identity_map.all_states = lambda : all_states
         for obj in objs:
             state = attributes.instance_state(obj)
             sess.identity_map.remove(state)
@@ -1314,7 +1373,8 @@ class DisposedStates(_base.MappedTest):
         global sess
         sess = create_session(**kwargs)
 
-        data = o1, o2, o3, o4, o5 = [T('t1'), T('t2'), T('t3'), T('t4'), T('t5')]
+        data = o1, o2, o3, o4, o5 = [T('t1'), T('t2'), T('t3'), T('t4'
+                ), T('t5')]
 
         sess.add_all(data)
 
@@ -1371,8 +1431,9 @@ class SessionInterface(testing.TestBase):
         return ok
 
     def _map_it(self, cls):
-        return mapper(cls, Table('t', sa.MetaData(),
-                                 Column('id', Integer, primary_key=True)))
+        return mapper(cls, Table('t', sa.MetaData(), Column('id',
+                      Integer, primary_key=True,
+                      test_needs_autoincrement=True)))
 
     @testing.uses_deprecated()
     def _test_instance_guards(self, user_arg):
@@ -1416,13 +1477,8 @@ class SessionInterface(testing.TestBase):
 
         raises_('refresh', user_arg)
 
-        raises_('save', user_arg)
-
-        raises_('save_or_update', user_arg)
-
-        raises_('update', user_arg)
-
-        instance_methods = self._public_session_methods() - self._class_methods
+        instance_methods = self._public_session_methods() \
+            - self._class_methods
 
         eq_(watchdog, instance_methods,
             watchdog.symmetric_difference(instance_methods))
@@ -1491,10 +1547,9 @@ class TLTransactionTest(engine_base.AltEngineTest, _base.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        Table('users', metadata,
-              Column('id', Integer, primary_key=True),
-              Column('name', String(20)),
-              test_needs_acid=True)
+        Table('users', metadata, Column('id', Integer,
+              primary_key=True, test_needs_autoincrement=True),
+              Column('name', String(20)), test_needs_acid=True)
 
     @classmethod
     def setup_classes(cls):
